@@ -8,8 +8,12 @@ AUTH=${AUTH:-noauth}
 SSL=${SSL:-nossl}
 URI=${URI:-}
 OS=$(uname -s | tr '[:upper:]' '[:lower:]')
-[ -z "$MARCH" ] && MARCH=$(uname -m | tr '[:upper:]' '[:lower:]')
+DNS=${DNS:-nodns}
 
+echo "COMPRESSORS='${COMPRESSORS}' CC='${CC}' AUTH=${AUTH} SSL=${SSL} URI=${URI} IPV4_ONLY=${IPV4_ONLY} VALGRIND=${VALGRIND} MONGOC_TEST_URI=${MONGOC_TEST_URI}"
+
+[ -z "$MARCH" ] && MARCH=$(uname -m | tr '[:upper:]' '[:lower:]')
+TEST_ARGS="-d -F test-results.json"
 
 if [ "$COMPRESSORS" != "nocompressors" ]; then
    export MONGOC_TEST_COMPRESSORS="$COMPRESSORS"
@@ -40,7 +44,16 @@ if [ "$IPV4_ONLY" != "on" ]; then
    export MONGOC_CHECK_IPV6="on"
 fi
 
+if [ "$DNS" != "nodns" ]; then
+   export MONGOC_TEST_DNS=on
+   TEST_ARGS="$TEST_ARGS -l /initial_dns_seedlist_discovery*"
+fi
+
 if [ "$CC" = "mingw" ]; then
+   if [ "$DNS" != "nodns" ]; then
+      echo "ERROR - DNS tests not implemented for MinGW yet"
+      exit 1
+   fi
    chmod +x test-libmongoc.exe
    cmd.exe /c .evergreen\\run-tests-mingw.bat
    exit 0
@@ -55,14 +68,10 @@ case "$OS" in
       ;;
 
    darwin)
+      sed -i'.bak' 's/\/data\/mci\/[a-z0-9]\{32\}\/mongoc/./g' test-libmongoc
       export DYLD_LIBRARY_PATH=".libs:src/libbson/.libs"
       ;;
 
-   sunos)
-      PATH="/opt/mongodbtoolchain/bin:$PATH"
-      export LD_LIBRARY_PATH="/opt/csw/lib/amd64/:.libs:src/libbson/.libs"
-      ;;
-
    *)
       #if test -f /tmp/drivers.keytab; then
          # See CDRIVER-2000
@@ -85,15 +94,20 @@ esac
 
 case "$OS" in
    cygwin*)
-      test-libmongoc.exe -d -F test-results.json
-      ;;
-
-   sunos)
-      gmake -o test-libmongoc test TEST_ARGS="--no-fork -d -F test-results.json"
+      test-libmongoc.exe $TEST_ARGS
       ;;
 
    *)
-      make -o test-libmongoc test TEST_ARGS="--no-fork -d -F test-results.json"
+      ulimit -c unlimited || true
+
+      if [ "$VALGRIND" = "on" ]; then
+         export MONGOC_TEST_SKIP_SLOW="on";
+         make valgrind;
+      else
+         TEST_ARGS="--no-fork $TEST_ARGS"
+         make -o test-libmongoc test TEST_ARGS="$TEST_ARGS"
+      fi
+
       ;;
 esac
 
