@@ -14,7 +14,7 @@ bulk1 (mongoc_collection_t *collection)
    bool ret;
    int i;
 
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
 
    for (i = 0; i < 10000; i++) {
       doc = BCON_NEW ("i", BCON_INT32 (i));
@@ -24,7 +24,7 @@ bulk1 (mongoc_collection_t *collection)
 
    ret = mongoc_bulk_operation_execute (bulk, &reply, &error);
 
-   str = bson_as_json (&reply, NULL);
+   str = bson_as_canonical_extended_json (&reply, NULL);
    printf ("%s\n", str);
    bson_free (str);
 
@@ -37,8 +37,7 @@ bulk1 (mongoc_collection_t *collection)
 }
 
 int
-main (int argc,
-      char *argv[])
+main (int argc, char *argv[])
 {
    mongoc_client_t *client;
    mongoc_collection_t *collection;
