@@ -17,18 +17,29 @@
 #ifndef MONGOC_CLIENT_POOL_PRIVATE_H
 #define MONGOC_CLIENT_POOL_PRIVATE_H
 
-#if !defined (MONGOC_COMPILATION)
+#if !defined(MONGOC_COMPILATION)
 #error "Only <mongoc.h> can be included directly."
 #endif
 
 #include <bson.h>
 
 #include "mongoc-client-pool.h"
+#include "mongoc-topology-description.h"
+#include "mongoc-topology-private.h"
 
 BSON_BEGIN_DECLS
 
-size_t 				  mongoc_client_pool_get_size   (mongoc_client_pool_t *pool);
-size_t              mongoc_client_pool_num_pushed (mongoc_client_pool_t *pool);
+/* for tests */
+void
+_mongoc_client_pool_set_stream_initiator (mongoc_client_pool_t *pool,
+                                          mongoc_stream_initiator_t si,
+                                          void *user_data);
+size_t
+mongoc_client_pool_get_size (mongoc_client_pool_t *pool);
+size_t
+mongoc_client_pool_num_pushed (mongoc_client_pool_t *pool);
+mongoc_topology_t *
+_mongoc_client_pool_get_topology (mongoc_client_pool_t *pool);
 
 BSON_END_DECLS
 
