@@ -17,12 +17,13 @@
 #ifndef MONGOC_GRIDFS_FILE_LIST_H
 #define MONGOC_GRIDFS_FILE_LIST_H
 
-#if !defined (MONGOC_INSIDE) && !defined (MONGOC_COMPILATION)
-# error "Only <mongoc.h> can be included directly."
+#if !defined(MONGOC_INSIDE) && !defined(MONGOC_COMPILATION)
+#error "Only <mongoc.h> can be included directly."
 #endif
 
 #include <bson.h>
 
+#include "mongoc-macros.h"
 #include "mongoc-gridfs-file.h"
 
 
@@ -32,10 +33,13 @@ BSON_BEGIN_DECLS
 typedef struct _mongoc_gridfs_file_list_t mongoc_gridfs_file_list_t;
 
 
-mongoc_gridfs_file_t *mongoc_gridfs_file_list_next    (mongoc_gridfs_file_list_t *list);
-void                  mongoc_gridfs_file_list_destroy (mongoc_gridfs_file_list_t *list);
-bool                  mongoc_gridfs_file_list_error   (mongoc_gridfs_file_list_t *list,
-                                                       bson_error_t              *error);
+MONGOC_EXPORT (mongoc_gridfs_file_t *)
+mongoc_gridfs_file_list_next (mongoc_gridfs_file_list_t *list);
+MONGOC_EXPORT (void)
+mongoc_gridfs_file_list_destroy (mongoc_gridfs_file_list_t *list);
+MONGOC_EXPORT (bool)
+mongoc_gridfs_file_list_error (mongoc_gridfs_file_list_t *list,
+                               bson_error_t *error);
 
 
 BSON_END_DECLS
