@@ -15,10 +15,9 @@
  */
 
 
-#include "mongoc-config.h"
 #include "mongoc-error.h"
 #include "mongoc-read-prefs-private.h"
-#include "mongoc-trace.h"
+#include "mongoc-trace-private.h"
 
 
 mongoc_read_prefs_t *
@@ -26,10 +25,10 @@ mongoc_read_prefs_new (mongoc_read_mode_t mode)
 {
    mongoc_read_prefs_t *read_prefs;
 
-   read_prefs = (mongoc_read_prefs_t *)bson_malloc0(sizeof *read_prefs);
+   read_prefs = (mongoc_read_prefs_t *) bson_malloc0 (sizeof *read_prefs);
    read_prefs->mode = mode;
-   bson_init(&read_prefs->tags);
-   read_prefs->max_staleness_ms = 0;  /* no maximum staleness */
+   bson_init (&read_prefs->tags);
+   read_prefs->max_staleness_seconds = MONGOC_NO_MAX_STALENESS;
 
    return read_prefs;
 }
@@ -44,7 +43,7 @@ mongoc_read_prefs_get_mode (const mongoc_read_prefs_t *read_prefs)
 
 void
 mongoc_read_prefs_set_mode (mongoc_read_prefs_t *read_prefs,
-                            mongoc_read_mode_t   mode)
+                            mongoc_read_mode_t mode)
 {
    BSON_ASSERT (read_prefs);
    BSON_ASSERT (mode <= MONGOC_READ_NEAREST);
@@ -62,24 +61,22 @@ mongoc_read_prefs_get_tags (const mongoc_read_prefs_t *read_prefs)
 
 
 void
-mongoc_read_prefs_set_tags (mongoc_read_prefs_t *read_prefs,
-                            const bson_t        *tags)
+mongoc_read_prefs_set_tags (mongoc_read_prefs_t *read_prefs, const bson_t *tags)
 {
    BSON_ASSERT (read_prefs);
 
-   bson_destroy(&read_prefs->tags);
+   bson_destroy (&read_prefs->tags);
 
    if (tags) {
-      bson_copy_to(tags, &read_prefs->tags);
+      bson_copy_to (tags, &read_prefs->tags);
    } else {
-      bson_init(&read_prefs->tags);
+      bson_init (&read_prefs->tags);
    }
 }
 
 
 void
-mongoc_read_prefs_add_tag (mongoc_read_prefs_t *read_prefs,
-                           const bson_t        *tag)
+mongoc_read_prefs_add_tag (mongoc_read_prefs_t *read_prefs, const bson_t *tag)
 {
    bson_t empty = BSON_INITIALIZER;
    char str[16];
@@ -98,25 +95,24 @@ mongoc_read_prefs_add_tag (mongoc_read_prefs_t *read_prefs,
 }
 
 
-#ifdef MONGOC_EXPERIMENTAL_FEATURES
-int32_t
-mongoc_read_prefs_get_max_staleness_ms (const mongoc_read_prefs_t *read_prefs)
+int64_t
+mongoc_read_prefs_get_max_staleness_seconds (
+   const mongoc_read_prefs_t *read_prefs)
 {
    BSON_ASSERT (read_prefs);
 
-   return read_prefs->max_staleness_ms;
+   return read_prefs->max_staleness_seconds;
 }
 
 
 void
-mongoc_read_prefs_set_max_staleness_ms (mongoc_read_prefs_t *read_prefs,
-                                        int32_t              max_staleness_ms)
+mongoc_read_prefs_set_max_staleness_seconds (mongoc_read_prefs_t *read_prefs,
+                                             int64_t max_staleness_seconds)
 {
    BSON_ASSERT (read_prefs);
 
-   read_prefs->max_staleness_ms = max_staleness_ms;
+   read_prefs->max_staleness_seconds = max_staleness_seconds;
 }
-#endif
 
 
 bool
@@ -125,15 +121,17 @@ mongoc_read_prefs_is_valid (const mongoc_read_prefs_t *read_prefs)
    BSON_ASSERT (read_prefs);
 
    /*
-    * Tags or maxStalenessMS are not supported with PRIMARY mode.
+    * Tags or maxStalenessSeconds are not supported with PRIMARY mode.
     */
    if (read_prefs->mode == MONGOC_READ_PRIMARY) {
-      if (!bson_empty(&read_prefs->tags) || read_prefs->max_staleness_ms > 0) {
+      if (!bson_empty (&read_prefs->tags) ||
+          read_prefs->max_staleness_seconds != MONGOC_NO_MAX_STALENESS) {
          return false;
       }
    }
 
-   if (read_prefs->max_staleness_ms < 0) {
+   if (read_prefs->max_staleness_seconds != MONGOC_NO_MAX_STALENESS &&
+       read_prefs->max_staleness_seconds <= 0) {
       return false;
    }
 
@@ -145,8 +143,8 @@ void
 mongoc_read_prefs_destroy (mongoc_read_prefs_t *read_prefs)
 {
    if (read_prefs) {
-      bson_destroy(&read_prefs->tags);
-      bson_free(read_prefs);
+      bson_destroy (&read_prefs->tags);
+      bson_free (read_prefs);
    }
 }
 
@@ -157,17 +155,17 @@ mongoc_read_prefs_copy (const mongoc_read_prefs_t *read_prefs)
    mongoc_read_prefs_t *ret = NULL;
 
    if (read_prefs) {
-      ret = mongoc_read_prefs_new(read_prefs->mode);
-      bson_copy_to(&read_prefs->tags, &ret->tags);
-      ret->max_staleness_ms = read_prefs->max_staleness_ms;
+      ret = mongoc_read_prefs_new (read_prefs->mode);
+      bson_copy_to (&read_prefs->tags, &ret->tags);
+      ret->max_staleness_seconds = read_prefs->max_staleness_seconds;
    }
 
    return ret;
 }
 
 
-static const char *
-_get_read_mode_string (mongoc_read_mode_t mode)
+const char *
+_mongoc_read_mode_as_str (mongoc_read_mode_t mode)
 {
    switch (mode) {
    case MONGOC_READ_PRIMARY:
@@ -190,17 +188,16 @@ _get_read_mode_string (mongoc_read_mode_t mode)
  * The driver must have discovered the server is a mongos.
  */
 static void
-_apply_read_preferences_mongos (const mongoc_read_prefs_t *read_prefs,
-                                const bson_t *query_bson,
-                                mongoc_apply_read_prefs_result_t *result /* OUT */)
+_apply_read_preferences_mongos (
+   const mongoc_read_prefs_t *read_prefs,
+   const bson_t *query_bson,
+   mongoc_assemble_query_result_t *result /* OUT */)
 {
    mongoc_read_mode_t mode;
    const bson_t *tags = NULL;
    bson_t child;
    const char *mode_str;
-#ifdef MONGOC_EXPERIMENTAL_FEATURES
-   int64_t max_staleness_ms;
-#endif
+   int64_t max_staleness_seconds;
 
    mode = mongoc_read_prefs_get_mode (read_prefs);
    if (read_prefs) {
@@ -238,55 +235,62 @@ _apply_read_preferences_mongos (const mongoc_read_prefs_t *read_prefs,
        *
        * This applies to commands, too.
        */
-      result->query_with_read_prefs = bson_new ();
+      result->assembled_query = bson_new ();
       result->query_owned = true;
 
       if (bson_has_field (query_bson, "$query")) {
-         bson_concat (result->query_with_read_prefs, query_bson);
+         bson_concat (result->assembled_query, query_bson);
       } else {
-         bson_append_document (result->query_with_read_prefs,
-                               "$query", 6, query_bson);
+         bson_append_document (
+            result->assembled_query, "$query", 6, query_bson);
       }
 
-      bson_append_document_begin (result->query_with_read_prefs,
-                                  "$readPreference", 15, &child);
-      mode_str = _get_read_mode_string (mode);
+      bson_append_document_begin (
+         result->assembled_query, "$readPreference", 15, &child);
+      mode_str = _mongoc_read_mode_as_str (mode);
       bson_append_utf8 (&child, "mode", 4, mode_str, -1);
       if (!bson_empty0 (tags)) {
          bson_append_array (&child, "tags", 4, tags);
       }
 
-#ifdef MONGOC_EXPERIMENTAL_FEATURES
-      max_staleness_ms = mongoc_read_prefs_get_max_staleness_ms (read_prefs);
-      if (max_staleness_ms > 0) {
-         bson_append_int64 (&child, "maxStalenessMS", 14, max_staleness_ms);
-      }
-#endif
+      max_staleness_seconds =
+         mongoc_read_prefs_get_max_staleness_seconds (read_prefs);
 
-      bson_append_document_end (result->query_with_read_prefs, &child);
+      if (max_staleness_seconds != MONGOC_NO_MAX_STALENESS) {
+         bson_append_int64 (
+            &child, "maxStalenessSeconds", 19, max_staleness_seconds);
+      }
+
+      bson_append_document_end (result->assembled_query, &child);
    }
 }
 
 /*
  *--------------------------------------------------------------------------
  *
- * apply_read_preferences --
+ * assemble_query --
  *
- *       Update @result based on @read prefs, following the Server Selection
+ *       Update @result based on @read_prefs, following the Server Selection
  *       Spec.
  *
  * Side effects:
- *       Sets @result->query_with_read_prefs and @result->flags.
+ *       Sets @result->assembled_query and @result->flags.
+ *
+ *  Note:
+ *       This function, the mongoc_assemble_query_result_t struct, and all
+ *       related functions are only used for find operations with OP_QUERY.
+ *       Remove them once we have implemented exhaust cursors with OP_MSG in
+ *       the server, and all previous server versions are EOL.
  *
  *--------------------------------------------------------------------------
  */
 
 void
-apply_read_preferences (const mongoc_read_prefs_t *read_prefs,
-                        const mongoc_server_stream_t *server_stream,
-                        const bson_t *query_bson,
-                        mongoc_query_flags_t initial_flags,
-                        mongoc_apply_read_prefs_result_t *result /* OUT */)
+assemble_query (const mongoc_read_prefs_t *read_prefs,
+                const mongoc_server_stream_t *server_stream,
+                const bson_t *query_bson,
+                mongoc_query_flags_t initial_flags,
+                mongoc_assemble_query_result_t *result /* OUT */)
 {
    mongoc_server_description_type_t server_type;
 
@@ -297,7 +301,7 @@ apply_read_preferences (const mongoc_read_prefs_t *read_prefs,
    BSON_ASSERT (result);
 
    /* default values */
-   result->query_with_read_prefs = (bson_t *) query_bson;
+   result->assembled_query = (bson_t *) query_bson;
    result->query_owned = false;
    result->flags = initial_flags;
 
@@ -348,25 +352,26 @@ apply_read_preferences (const mongoc_read_prefs_t *read_prefs,
 
 
 void
-apply_read_prefs_result_cleanup (mongoc_apply_read_prefs_result_t *result)
+assemble_query_result_cleanup (mongoc_assemble_query_result_t *result)
 {
    ENTRY;
 
    BSON_ASSERT (result);
 
    if (result->query_owned) {
-      bson_destroy (result->query_with_read_prefs);
+      bson_destroy (result->assembled_query);
    }
 
    EXIT;
 }
 
 bool
-_mongoc_read_prefs_validate (const mongoc_read_prefs_t *read_prefs, 
-                             bson_error_t              *error)
+_mongoc_read_prefs_validate (const mongoc_read_prefs_t *read_prefs,
+                             bson_error_t *error)
 {
    if (read_prefs && !mongoc_read_prefs_is_valid (read_prefs)) {
-      bson_set_error (error, MONGOC_ERROR_COMMAND, 
+      bson_set_error (error,
+                      MONGOC_ERROR_COMMAND,
                       MONGOC_ERROR_COMMAND_INVALID_ARG,
                       "Invalid mongoc_read_prefs_t");
       return false;
