@@ -23,24 +23,38 @@
 #include "../TestSuite.h"
 
 
-static bool is_command_ns (const char *ns);
+static bool
+is_command_ns (const char *ns);
 
-static void request_from_query (request_t *request, const mongoc_rpc_t *rpc);
+static void
+request_from_query (request_t *request, const mongoc_rpc_t *rpc);
 
-static void request_from_insert (request_t *request, const mongoc_rpc_t *rpc);
+static void
+request_from_insert (request_t *request, const mongoc_rpc_t *rpc);
 
-static void request_from_update (request_t *request, const mongoc_rpc_t *rpc);
+static void
+request_from_update (request_t *request, const mongoc_rpc_t *rpc);
 
-static void request_from_delete (request_t *request, const mongoc_rpc_t *rpc);
+static void
+request_from_delete (request_t *request, const mongoc_rpc_t *rpc);
 
-static void request_from_killcursors (request_t *request, const mongoc_rpc_t *rpc);
+static void
+request_from_killcursors (request_t *request, const mongoc_rpc_t *rpc);
 
-static void request_from_getmore (request_t *request, const mongoc_rpc_t *rpc);
+static void
+request_from_getmore (request_t *request, const mongoc_rpc_t *rpc);
 
-static char *query_flags_str (uint32_t flags);
-static char *insert_flags_str (uint32_t flags);
-static char *update_flags_str (uint32_t flags);
-static char *delete_flags_str (uint32_t flags);
+static void
+request_from_op_msg (request_t *request, const mongoc_rpc_t *rpc);
+
+static char *
+query_flags_str (uint32_t flags);
+static char *
+insert_flags_str (uint32_t flags);
+static char *
+update_flags_str (uint32_t flags);
+static char *
+delete_flags_str (uint32_t flags);
 
 request_t *
 request_new (const mongoc_buffer_t *buffer,
@@ -50,19 +64,17 @@ request_new (const mongoc_buffer_t *buffer,
              uint16_t client_port,
              sync_queue_t *replies)
 {
-   request_t *request = (request_t *)bson_malloc0 (sizeof *request);
+   request_t *request = (request_t *) bson_malloc0 (sizeof *request);
    uint8_t *data;
 
-   data = (uint8_t *)bson_malloc ((size_t)msg_len);
+   data = (uint8_t *) bson_malloc ((size_t) msg_len);
    memcpy (data, buffer->data + buffer->off, (size_t) msg_len);
    request->data = data;
    request->data_len = (size_t) msg_len;
    request->replies = replies;
 
-   if (!_mongoc_rpc_scatter (&request->request_rpc, data,
-                             (size_t) msg_len)) {
-      MONGOC_WARNING ("%s():%d: %s", BSON_FUNC, __LINE__,
-                      "Failed to scatter");
+   if (!_mongoc_rpc_scatter (&request->request_rpc, data, (size_t) msg_len)) {
+      MONGOC_WARNING ("%s():%d: %s", BSON_FUNC, __LINE__, "Failed to scatter");
       bson_free (data);
       bson_free (request);
       return NULL;
@@ -77,6 +89,8 @@ request_new (const mongoc_buffer_t *buffer,
    _mongoc_array_init (&request->docs, sizeof (bson_t *));
 
    switch (request->opcode) {
+   case MONGOC_OPCODE_COMPRESSED:
+      break;
    case MONGOC_OPCODE_QUERY:
       request_from_query (request, &request->request_rpc);
       break;
@@ -101,8 +115,11 @@ request_new (const mongoc_buffer_t *buffer,
       request_from_delete (request, &request->request_rpc);
       break;
 
-   case MONGOC_OPCODE_REPLY:
    case MONGOC_OPCODE_MSG:
+      request_from_op_msg (request, &request->request_rpc);
+      break;
+
+   case MONGOC_OPCODE_REPLY:
    default:
       fprintf (stderr, "Unimplemented opcode %d\n", request->opcode);
       abort ();
@@ -112,19 +129,17 @@ request_new (const mongoc_buffer_t *buffer,
 }
 
 const bson_t *
-request_get_doc (const request_t *request,
-                 int n)
+request_get_doc (const request_t *request, int n)
 {
    return _mongoc_array_index (&request->docs, const bson_t *, n);
 }
 
 bool
-request_matches_flags (const request_t *request,
-                       mongoc_query_flags_t flags)
+request_matches_flags (const request_t *request, mongoc_query_flags_t flags)
 {
    const mongoc_rpc_t *rpc;
 
-   assert (request);
+   BSON_ASSERT (request);
    rpc = &request->request_rpc;
 
    if (rpc->query.flags != flags) {
@@ -150,84 +165,99 @@ request_matches_query (const request_t *request,
 {
    const mongoc_rpc_t *rpc;
    const bson_t *doc;
+   const bson_t *doc2;
+   char *doc_as_json;
    bool n_return_equal;
+   bool ret = false;
 
-   assert (request);
+   BSON_ASSERT (request);
    rpc = &request->request_rpc;
 
-   assert (request->docs.len <= 2);
+   BSON_ASSERT (request->docs.len <= 2);
+
+   if (request->docs.len) {
+      doc = request_get_doc (request, 0);
+      doc_as_json = bson_as_json (doc, NULL);
+   } else {
+      doc = NULL;
+      doc_as_json = NULL;
+   }
+
+   if (!match_json (
+          doc, is_command, __FILE__, __LINE__, BSON_FUNC, query_json)) {
+      /* match_json has logged the err */
+      goto done;
+   }
+
+   if (request->docs.len > 1) {
+      doc2 = request_get_doc (request, 1);
+   } else {
+      doc2 = NULL;
+   }
+
+   if (!match_json (doc2, false, __FILE__, __LINE__, BSON_FUNC, fields_json)) {
+      /* match_json has logged the err */
+      goto done;
+   }
 
    if (request->is_command && !is_command) {
-      test_error ("expected query, got command");
-      return false;
+      test_error ("expected query, got command: %s", doc_as_json);
+      goto done;
    }
 
    if (!request->is_command && is_command) {
-      test_error ("expected command, got query");
-      return false;
+      test_error ("expected command, got query: %s", doc_as_json);
+      goto done;
    }
 
    if (request->opcode != MONGOC_OPCODE_QUERY) {
-      test_error ("request's opcode does not match QUERY");
-      return false;
+      test_error ("request's opcode does not match QUERY: %s", doc_as_json);
+      goto done;
    }
 
-   if (strcmp (rpc->query.collection, ns)) {
-      test_error ("request's namespace is '%s', expected '%s'",
-                    request->request_rpc.query.collection, ns);
-      return false;
+   if (0 != strcmp (rpc->query.collection, ns)) {
+      test_error ("request's namespace is '%s', expected '%s': %s",
+                  request->request_rpc.query.collection,
+                  ns,
+                  doc_as_json);
+      goto done;
    }
 
    if (!request_matches_flags (request, flags)) {
-      return false;
+      test_error ("%s", doc_as_json);
+      goto done;
    }
 
    if (rpc->query.skip != skip) {
-      test_error ("requests's skip = %d, expected %d",
-                    rpc->query.skip, skip);
-      return false;
+      test_error ("requests's skip = %d, expected %d: %s",
+                  rpc->query.skip,
+                  skip,
+                  doc_as_json);
+      goto done;
    }
 
    n_return_equal = (rpc->query.n_return == n_return);
 
-   if (! n_return_equal && abs (rpc->query.n_return) == 1) {
+   if (!n_return_equal && abs (rpc->query.n_return) == 1) {
       /* quirk: commands from mongoc_client_command_simple have n_return 1,
        * from mongoc_topology_scanner_t have n_return -1
        */
       n_return_equal = abs (rpc->query.n_return) == n_return;
    }
 
-   if (! n_return_equal) {
-      test_error ("requests's n_return = %d, expected %d",
-                  rpc->query.n_return, n_return);
-      return false;
+   if (!n_return_equal) {
+      test_error ("requests's n_return = %d, expected %d: %s",
+                  rpc->query.n_return,
+                  n_return,
+                  doc_as_json);
+      goto done;
    }
 
-   if (request->docs.len) {
-      doc = request_get_doc(request, 0);
-   } else {
-      doc = NULL;
-   }
+   ret = true;
 
-   if (!match_json (doc, is_command, __FILE__, __LINE__,
-                    BSON_FUNC, query_json)) {
-      /* match_json has logged the err */
-      return false;
-   }
-
-   if (request->docs.len > 1) {
-      doc = request_get_doc (request, 1);
-   } else {
-      doc = NULL;
-   }
-
-   if (!match_json (doc, false,
-                    __FILE__, __LINE__, BSON_FUNC, fields_json)) {
-      /* match_json has logged the err */
-      return false;
-   }
-
-   return true;
+done:
+   bson_free (doc_as_json);
+   return ret;
 }
 
 
@@ -241,17 +271,19 @@ request_matches_insert (const request_t *request,
    const mongoc_rpc_t *rpc;
    const bson_t *doc;
 
-   assert (request);
+   BSON_ASSERT (request);
    rpc = &request->request_rpc;
 
    if (request->opcode != MONGOC_OPCODE_INSERT) {
-      test_error ("request's opcode does not match INSERT");
+      test_error ("request's opcode does not match INSERT, got: %d",
+                  request->opcode);
       return false;
    }
 
    if (strcmp (rpc->insert.collection, ns)) {
       test_error ("insert's namespace is '%s', expected '%s'",
-                  request->request_rpc.get_more.collection, ns);
+                  request->request_rpc.get_more.collection,
+                  ns);
       return false;
    }
 
@@ -262,8 +294,8 @@ request_matches_insert (const request_t *request,
       return false;
    }
 
-   ASSERT_CMPINT ((int)request->docs.len, ==, 1);
-   doc = request_get_doc(request, 0);
+   ASSERT_CMPINT ((int) request->docs.len, ==, 1);
+   doc = request_get_doc (request, 0);
    if (!match_json (doc, false, __FILE__, __LINE__, BSON_FUNC, doc_json)) {
       return false;
    }
@@ -281,17 +313,19 @@ request_matches_bulk_insert (const request_t *request,
 {
    const mongoc_rpc_t *rpc;
 
-   assert (request);
+   BSON_ASSERT (request);
    rpc = &request->request_rpc;
 
    if (request->opcode != MONGOC_OPCODE_INSERT) {
-      test_error ("request's opcode does not match INSERT");
+      test_error ("request's opcode does not match INSERT, got: %d",
+                  request->opcode);
       return false;
    }
 
    if (strcmp (rpc->insert.collection, ns)) {
       test_error ("insert's namespace is '%s', expected '%s'",
-                  request->request_rpc.get_more.collection, ns);
+                  request->request_rpc.get_more.collection,
+                  ns);
       return false;
    }
 
@@ -302,9 +336,9 @@ request_matches_bulk_insert (const request_t *request,
       return false;
    }
 
-   if ((int)request->docs.len != n) {
-      test_error ("expected %d docs inserted, got %d",
-                  n, (int)request->docs.len);
+   if ((int) request->docs.len != n) {
+      test_error (
+         "expected %d docs inserted, got %d", n, (int) request->docs.len);
       return false;
    }
 
@@ -323,17 +357,19 @@ request_matches_update (const request_t *request,
    const mongoc_rpc_t *rpc;
    const bson_t *doc;
 
-   assert (request);
+   BSON_ASSERT (request);
    rpc = &request->request_rpc;
 
    if (request->opcode != MONGOC_OPCODE_UPDATE) {
-      test_error ("request's opcode does not match UPDATE");
+      test_error ("request's opcode does not match UPDATE, got: %d",
+                  request->opcode);
       return false;
    }
 
    if (strcmp (rpc->update.collection, ns)) {
       test_error ("update's namespace is '%s', expected '%s'",
-                  request->request_rpc.update.collection, ns);
+                  request->request_rpc.update.collection,
+                  ns);
       return false;
    }
 
@@ -344,13 +380,13 @@ request_matches_update (const request_t *request,
       return false;
    }
 
-   ASSERT_CMPINT ((int)request->docs.len, ==, 2);
-   doc = request_get_doc(request, 0);
+   ASSERT_CMPINT ((int) request->docs.len, ==, 2);
+   doc = request_get_doc (request, 0);
    if (!match_json (doc, false, __FILE__, __LINE__, BSON_FUNC, selector_json)) {
       return false;
    }
 
-   doc = request_get_doc(request, 1);
+   doc = request_get_doc (request, 1);
    if (!match_json (doc, false, __FILE__, __LINE__, BSON_FUNC, update_json)) {
       return false;
    }
@@ -369,17 +405,19 @@ request_matches_delete (const request_t *request,
    const mongoc_rpc_t *rpc;
    const bson_t *doc;
 
-   assert (request);
+   BSON_ASSERT (request);
    rpc = &request->request_rpc;
 
    if (request->opcode != MONGOC_OPCODE_DELETE) {
-      test_error ("request's opcode does not match DELETE");
+      test_error ("request's opcode does not match DELETE, got: %d",
+                  request->opcode);
       return false;
    }
 
    if (strcmp (rpc->delete_.collection, ns)) {
       test_error ("delete's namespace is '%s', expected '%s'",
-                  request->request_rpc.delete_.collection, ns);
+                  request->request_rpc.delete_.collection,
+                  ns);
       return false;
    }
 
@@ -390,8 +428,8 @@ request_matches_delete (const request_t *request,
       return false;
    }
 
-   ASSERT_CMPINT ((int)request->docs.len, ==, 1);
-   doc = request_get_doc(request, 0);
+   ASSERT_CMPINT ((int) request->docs.len, ==, 1);
+   doc = request_get_doc (request, 0);
    if (!match_json (doc, false, __FILE__, __LINE__, BSON_FUNC, selector_json)) {
       return false;
    }
@@ -409,29 +447,33 @@ request_matches_getmore (const request_t *request,
 {
    const mongoc_rpc_t *rpc;
 
-   assert (request);
+   BSON_ASSERT (request);
    rpc = &request->request_rpc;
 
    if (request->opcode != MONGOC_OPCODE_GET_MORE) {
-      test_error ("request's opcode does not match GET_MORE");
+      test_error ("request's opcode does not match GET_MORE, got: %d",
+                  request->opcode);
       return false;
    }
 
    if (strcmp (rpc->get_more.collection, ns)) {
       test_error ("request's namespace is '%s', expected '%s'",
-                  request->request_rpc.get_more.collection, ns);
+                  request->request_rpc.get_more.collection,
+                  ns);
       return false;
    }
 
    if (rpc->get_more.n_return != n_return) {
       test_error ("requests's n_return = %d, expected %d",
-                  rpc->get_more.n_return, n_return);
+                  rpc->get_more.n_return,
+                  n_return);
       return false;
    }
 
    if (rpc->get_more.cursor_id != cursor_id) {
       test_error ("requests's cursor_id = %" PRId64 ", expected %" PRId64,
-                  rpc->get_more.cursor_id, cursor_id);
+                  rpc->get_more.cursor_id,
+                  cursor_id);
       return false;
    }
 
@@ -441,16 +483,16 @@ request_matches_getmore (const request_t *request,
 
 /* TODO: take file, line, function params from caller, wrap in macro */
 bool
-request_matches_kill_cursors (const request_t *request,
-                              int64_t cursor_id)
+request_matches_kill_cursors (const request_t *request, int64_t cursor_id)
 {
    const mongoc_rpc_t *rpc;
 
-   assert (request);
+   BSON_ASSERT (request);
    rpc = &request->request_rpc;
 
    if (request->opcode != MONGOC_OPCODE_KILL_CURSORS) {
-      test_error ("request's opcode does not match KILL_CURSORS");
+      test_error ("request's opcode does not match KILL_CURSORS, got: %d",
+                  request->opcode);
       return false;
    }
 
@@ -462,7 +504,66 @@ request_matches_kill_cursors (const request_t *request,
 
    if (rpc->kill_cursors.cursors[0] != cursor_id) {
       test_error ("request's cursor_id %" PRId64 ", expected %" PRId64,
-                  rpc->kill_cursors.cursors[0], cursor_id);
+                  rpc->kill_cursors.cursors[0],
+                  cursor_id);
+      return false;
+   }
+
+   return true;
+}
+
+
+bool
+request_matches_msgv (const request_t *request, uint32_t flags, va_list *args)
+{
+   const bson_t *doc;
+   const bson_t *pattern;
+   bool is_command_doc;
+   int i;
+
+   BSON_ASSERT (request);
+   if (request->opcode != MONGOC_OPCODE_MSG) {
+      test_error ("%s", "request's opcode does not match OP_MSG");
+   }
+
+   BSON_ASSERT (request->docs.len >= 1);
+
+   i = 0;
+   while ((pattern = va_arg (*args, const bson_t *))) {
+      /* make sure the pattern is reasonable, e.g. that we didn't pass a string
+       * instead of a bson_t* by mistake */
+      BSON_ASSERT (bson_validate (
+         pattern, BSON_VALIDATE_EMPTY_KEYS | BSON_VALIDATE_UTF8, NULL));
+
+      if (i > request->docs.len) {
+         fprintf (stderr,
+                  "Expected at least %d documents in request, got %zu\n",
+                  i,
+                  request->docs.len);
+         return false;
+      }
+
+      doc = request_get_doc (request, i);
+      /* pass is_command=true for first doc, including "find" command */
+      is_command_doc = (i == 0);
+      match_bson (doc, pattern, is_command_doc);
+      i++;
+   }
+
+   if (i < request->docs.len) {
+      fprintf (stderr,
+               "Expected %d documents in request, got %zu\n",
+               i,
+               request->docs.len);
+      return false;
+   }
+
+
+   if (flags != request->request_rpc.msg.flags) {
+      fprintf (stderr,
+               "Expected OP_MSG flags %u, got %u\n",
+               flags,
+               request->request_rpc.msg.flags);
       return false;
    }
 
@@ -603,19 +704,18 @@ query_flags_str (uint32_t flags)
                break;
             case MONGOC_QUERY_NONE:
             default:
-               assert (false);
+               BSON_ASSERT (false);
             }
          }
       }
    }
 
-   return bson_string_free (str, false);  /* detach buffer */
+   return bson_string_free (str, false); /* detach buffer */
 }
 
 
 static void
-request_from_query (request_t *request,
-                    const mongoc_rpc_t *rpc)
+request_from_query (request_t *request, const mongoc_rpc_t *rpc)
 {
    int32_t len;
    bson_t *query;
@@ -627,7 +727,7 @@ request_from_query (request_t *request,
    memcpy (&len, rpc->query.query, 4);
    len = BSON_UINT32_FROM_LE (len);
    query = bson_new_from_data (rpc->query.query, (size_t) len);
-   assert (query);
+   BSON_ASSERT (query);
    _mongoc_array_append_val (&request->docs, query);
 
    bson_string_append_printf (query_as_str, "%s ", rpc->query.collection);
@@ -638,7 +738,8 @@ request_from_query (request_t *request,
       if (bson_iter_init (&iter, query) && bson_iter_next (&iter)) {
          request->command_name = bson_strdup (bson_iter_key (&iter));
       } else {
-         fprintf (stderr, "WARNING: no command name for %s\n",
+         fprintf (stderr,
+                  "WARNING: no command name for %s\n",
                   request->request_rpc.query.collection);
       }
    }
@@ -651,7 +752,7 @@ request_from_query (request_t *request,
       memcpy (&len, rpc->query.fields, 4);
       len = BSON_UINT32_FROM_LE (len);
       fields = bson_new_from_data (rpc->query.fields, (size_t) len);
-      assert (fields);
+      BSON_ASSERT (fields);
       _mongoc_array_append_val (&request->docs, fields);
 
       str = bson_as_json (fields, NULL);
@@ -667,13 +768,13 @@ request_from_query (request_t *request,
    bson_free (str);
 
    if (rpc->query.skip) {
-      bson_string_append_printf (query_as_str, " skip=%d",
-                                 (int) rpc->query.skip);
+      bson_string_append_printf (
+         query_as_str, " skip=%d", (int) rpc->query.skip);
    }
 
    if (rpc->query.n_return) {
-      bson_string_append_printf (query_as_str, " n_return=%d",
-                                 (int) rpc->query.n_return);
+      bson_string_append_printf (
+         query_as_str, " n_return=%d", (int) rpc->query.n_return);
    }
 
    request->as_str = bson_string_free (query_as_str, false);
@@ -692,7 +793,7 @@ insert_flags_str (uint32_t flags)
 
 
 static uint32_t
-length_prefix (void *data)
+length_prefix (const uint8_t *data)
 {
    uint32_t len_le;
 
@@ -703,12 +804,11 @@ length_prefix (void *data)
 
 
 static void
-request_from_insert (request_t *request,
-                     const mongoc_rpc_t *rpc)
+request_from_insert (request_t *request, const mongoc_rpc_t *rpc)
 {
-   uint8_t *pos = (uint8_t *)request->request_rpc.insert.documents->iov_base;
+   uint8_t *pos = (uint8_t *) request->request_rpc.insert.documents->iov_base;
    uint8_t *end = request->data + request->data_len;
-   bson_string_t *insert_as_str = bson_string_new("OP_INSERT");
+   bson_string_t *insert_as_str = bson_string_new ("OP_INSERT");
    bson_t *doc;
    size_t n_documents;
    size_t i;
@@ -717,25 +817,24 @@ request_from_insert (request_t *request,
    while (pos < end) {
       uint32_t len = length_prefix (pos);
       doc = bson_new_from_data (pos, len);
-      assert (doc);
+      BSON_ASSERT (doc);
       _mongoc_array_append_val (&request->docs, doc);
       pos += len;
    }
 
    n_documents = request->docs.len;
 
-   bson_string_append_printf (insert_as_str, " %d ", (int)n_documents);
+   bson_string_append_printf (insert_as_str, " %d ", (int) n_documents);
 
    for (i = 0; i < n_documents; i++) {
       str = bson_as_json (request_get_doc (request, (int) i), NULL);
-      assert (str);
+      BSON_ASSERT (str);
       bson_string_append (insert_as_str, str);
       bson_free (str);
 
       if (i < n_documents - 1) {
          bson_string_append (insert_as_str, ", ");
       }
-
    }
 
    bson_string_append (insert_as_str, " flags=");
@@ -777,19 +876,18 @@ update_flags_str (uint32_t flags)
                break;
             case MONGOC_UPDATE_NONE:
             default:
-               assert (false);
+               BSON_ASSERT (false);
             }
          }
       }
    }
 
-   return bson_string_free (str, false);  /* detach buffer */
+   return bson_string_free (str, false); /* detach buffer */
 }
 
 
 static void
-request_from_update (request_t *request,
-                     const mongoc_rpc_t *rpc)
+request_from_update (request_t *request, const mongoc_rpc_t *rpc)
 {
    int32_t len;
    bson_t *doc;
@@ -799,7 +897,7 @@ request_from_update (request_t *request,
    memcpy (&len, rpc->update.selector, 4);
    len = BSON_UINT32_FROM_LE (len);
    doc = bson_new_from_data (rpc->update.selector, (size_t) len);
-   assert (doc);
+   BSON_ASSERT (doc);
    _mongoc_array_append_val (&request->docs, doc);
 
    str = bson_as_json (doc, NULL);
@@ -807,11 +905,11 @@ request_from_update (request_t *request,
    bson_free (str);
 
    bson_string_append (update_as_str, ", ");
-   
+
    memcpy (&len, rpc->update.update, 4);
    len = BSON_UINT32_FROM_LE (len);
    doc = bson_new_from_data (rpc->update.update, (size_t) len);
-   assert (doc);
+   BSON_ASSERT (doc);
    _mongoc_array_append_val (&request->docs, doc);
 
    str = bson_as_json (doc, NULL);
@@ -840,8 +938,7 @@ delete_flags_str (uint32_t flags)
 
 
 static void
-request_from_delete (request_t *request,
-                     const mongoc_rpc_t *rpc)
+request_from_delete (request_t *request, const mongoc_rpc_t *rpc)
 {
    int32_t len;
    bson_t *doc;
@@ -851,7 +948,7 @@ request_from_delete (request_t *request,
    memcpy (&len, rpc->delete_.selector, 4);
    len = BSON_UINT32_FROM_LE (len);
    doc = bson_new_from_data (rpc->delete_.selector, (size_t) len);
-   assert (doc);
+   BSON_ASSERT (doc);
    _mongoc_array_append_val (&request->docs, doc);
 
    str = bson_as_json (doc, NULL);
@@ -869,23 +966,107 @@ request_from_delete (request_t *request,
 
 
 static void
-request_from_killcursors (request_t *request,
-                          const mongoc_rpc_t *rpc)
+request_from_killcursors (request_t *request, const mongoc_rpc_t *rpc)
 {
    /* protocol allows multiple cursor ids but we only implement one */
-   assert (rpc->kill_cursors.n_cursors == 1);
+   BSON_ASSERT (rpc->kill_cursors.n_cursors == 1);
    request->as_str = bson_strdup_printf ("OP_KILLCURSORS %" PRId64,
                                          rpc->kill_cursors.cursors[0]);
 }
 
 
 static void
-request_from_getmore (request_t *request,
-                      const mongoc_rpc_t *rpc)
+request_from_getmore (request_t *request, const mongoc_rpc_t *rpc)
 {
-   request->as_str = bson_strdup_printf (
-      "OP_GETMORE %s %" PRId64 " n_return=%d",
-      rpc->get_more.collection,
-      rpc->get_more.cursor_id,
-      rpc->get_more.n_return);
+   request->as_str =
+      bson_strdup_printf ("OP_GETMORE %s %" PRId64 " n_return=%d",
+                          rpc->get_more.collection,
+                          rpc->get_more.cursor_id,
+                          rpc->get_more.n_return);
+}
+
+
+static void
+parse_op_msg_doc (request_t *request,
+                  const uint8_t *data,
+                  int32_t len,
+                  bson_string_t *msg_as_str)
+{
+   int32_t data_len;
+   int32_t doc_len;
+   bson_t *doc;
+   const uint8_t *pos;
+   char *str;
+
+   if (len == -1) {
+      data_len = length_prefix (data);
+   } else {
+      data_len = len;
+   }
+
+   pos = data;
+   while (pos < data + data_len) {
+      if (pos > data) {
+         bson_string_append (msg_as_str, ", ");
+      }
+
+      doc_len = length_prefix (pos);
+      doc = bson_new_from_data (pos, (size_t) doc_len);
+      BSON_ASSERT (doc);
+      _mongoc_array_append_val (&request->docs, doc);
+
+      str = bson_as_json (doc, NULL);
+      bson_string_append (msg_as_str, str);
+      bson_free (str);
+
+      pos += doc_len;
+   }
+}
+
+
+static void
+request_from_op_msg (request_t *request, const mongoc_rpc_t *rpc)
+{
+   const mongoc_rpc_section_t *section;
+   int32_t section_no;
+   const bson_t *doc;
+   bson_iter_t iter;
+   bson_string_t *msg_as_str = bson_string_new ("OP_MSG");
+
+   for (section_no = 0; section_no < rpc->msg.n_sections; section_no++) {
+      bson_string_append (msg_as_str, (section_no > 0 ? ", " : " "));
+      section = &rpc->msg.sections[section_no];
+      switch (section->payload_type) {
+      case 0:
+         /* a single BSON document */
+         parse_op_msg_doc (
+            request, section->payload.bson_document, -1, msg_as_str);
+         break;
+      case 1:
+         /* a sequence of BSON documents */
+         bson_string_append (msg_as_str, section->payload.sequence.identifier);
+         bson_string_append (msg_as_str, ": [");
+         parse_op_msg_doc (request,
+                           section->payload.sequence.bson_documents,
+                           section->payload.sequence.size,
+                           msg_as_str);
+         bson_string_append (msg_as_str, "]");
+         break;
+      default:
+         fprintf (
+            stderr, "Unimplemented payload type %d\n", section->payload_type);
+         abort ();
+      }
+   }
+
+   request->as_str = bson_string_free (msg_as_str, false);
+   request->is_command = true; /* true for all OP_MSG requests */
+
+   if (request->docs.len) {
+      doc = request_get_doc (request, 0);
+      bson_iter_init (&iter, doc);
+      if (bson_iter_next (&iter)) {
+         request->command_name = bson_strdup (bson_iter_key (&iter));
+      }
+   }
 }
