@@ -22,23 +22,22 @@ typedef struct ssl_test_result {
    unsigned long ssl_err;
 } ssl_test_result_t;
 
-typedef struct ssl_test_data
-{
-   mongoc_ssl_opt_t    *client;
-   mongoc_ssl_opt_t    *server;
-   ssl_test_behavior_t  behavior;
-   int64_t              handshake_stall_ms;
-   const char          *host;
-   unsigned short       server_port;
-   mongoc_cond_t        cond;
-   mongoc_mutex_t       cond_mutex;
-   ssl_test_result_t   *client_result;
-   ssl_test_result_t   *server_result;
+typedef struct ssl_test_data {
+   mongoc_ssl_opt_t *client;
+   mongoc_ssl_opt_t *server;
+   ssl_test_behavior_t behavior;
+   int64_t handshake_stall_ms;
+   const char *host;
+   unsigned short server_port;
+   mongoc_cond_t cond;
+   mongoc_mutex_t cond_mutex;
+   ssl_test_result_t *client_result;
+   ssl_test_result_t *server_result;
 } ssl_test_data_t;
 
 void
-ssl_test (mongoc_ssl_opt_t  *client,
-          mongoc_ssl_opt_t  *server,
-          const char        *host,
+ssl_test (mongoc_ssl_opt_t *client,
+          mongoc_ssl_opt_t *server,
+          const char *host,
           ssl_test_result_t *client_result,
           ssl_test_result_t *server_result);
