@@ -11,25 +11,23 @@
 #include "mock_server/mock-rs.h"
 
 
-typedef struct
-{
-   const char           *filter;
-   bson_t               *filter_bson;
-   const char           *opts;
-   bson_t               *opts_bson;
-   mongoc_read_prefs_t  *read_prefs;
-   const char           *expected_find_command;
-   const char           *expected_op_query;
-   const char           *expected_op_query_projection;
-   int32_t               expected_n_return;
-   mongoc_query_flags_t  expected_flags;
-   uint32_t              expected_skip;
+typedef struct {
+   const char *filter;
+   bson_t *filter_bson;
+   const char *opts;
+   bson_t *opts_bson;
+   mongoc_read_prefs_t *read_prefs;
+   const char *expected_find_command;
+   const char *expected_op_query;
+   const char *expected_op_query_projection;
+   int32_t expected_n_return;
+   mongoc_query_flags_t expected_flags;
+   uint32_t expected_skip;
 } test_collection_find_with_opts_t;
 
 
 typedef request_t *(*check_request_fn_t) (
-   mock_server_t *server,
-   test_collection_find_with_opts_t *test_data);
+   mock_server_t *server, test_collection_find_with_opts_t *test_data);
 
 
 /*--------------------------------------------------------------------------
@@ -47,9 +45,9 @@ typedef request_t *(*check_request_fn_t) (
 static void
 _test_collection_op_query_or_find_command (
    test_collection_find_with_opts_t *test_data,
-   check_request_fn_t                check_request_fn,
-   const char                       *reply_json,
-   int32_t                           max_wire_version)
+   check_request_fn_t check_request_fn,
+   const char *reply_json,
+   int32_t max_wire_version)
 {
    mock_server_t *server;
    mongoc_client_t *client;
@@ -86,7 +84,7 @@ _test_collection_op_query_or_find_command (
 
 
 static request_t *
-_check_op_query (mock_server_t                    *server,
+_check_op_query (mock_server_t *server,
                  test_collection_find_with_opts_t *test_data)
 {
    mongoc_query_flags_t flags;
@@ -100,14 +98,14 @@ _check_op_query (mock_server_t                    *server,
    /* Server Selection Spec: all queries to standalone set slaveOk. */
    flags = test_data->expected_flags | MONGOC_QUERY_SLAVE_OK;
 
-   request = mock_server_receives_query (
-      server,
-      "db.collection",
-      flags,
-      test_data->expected_skip,
-      test_data->expected_n_return,
-      test_data->expected_op_query,
-      test_data->expected_op_query_projection);
+   request =
+      mock_server_receives_query (server,
+                                  "db.collection",
+                                  flags,
+                                  test_data->expected_skip,
+                                  test_data->expected_n_return,
+                                  test_data->expected_op_query,
+                                  test_data->expected_op_query_projection);
 
    ASSERT (request);
 
@@ -130,15 +128,13 @@ _check_op_query (mock_server_t                    *server,
 static void
 _test_collection_op_query (test_collection_find_with_opts_t *test_data)
 {
-   _test_collection_op_query_or_find_command (test_data,
-                                              _check_op_query,
-                                              "{}",
-                                              3 /* wire version */);
+   _test_collection_op_query_or_find_command (
+      test_data, _check_op_query, "{}", 3 /* wire version */);
 }
 
 
 static request_t *
-_check_find_command (mock_server_t                    *server,
+_check_find_command (mock_server_t *server,
                      test_collection_find_with_opts_t *test_data)
 {
    /* Server Selection Spec: all queries to standalone set slaveOk.
@@ -146,8 +142,8 @@ _check_find_command (mock_server_t                    *server,
     * Find, getMore And killCursors Commands Spec: "When sending a find command
     * rather than a legacy OP_QUERY find only the slaveOk flag is honored".
     */
-   return mock_server_receives_command (server, "db", MONGOC_QUERY_SLAVE_OK,
-                                        test_data->expected_find_command);
+   return mock_server_receives_command (
+      server, "db", MONGOC_QUERY_SLAVE_OK, test_data->expected_find_command);
 }
 
 
@@ -182,7 +178,7 @@ _test_collection_find_with_opts (test_collection_find_with_opts_t *test_data)
 static void
 test_dollar_or (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.filter = "{'$or': [{'_id': 1}]}";
    test_data.expected_op_query = " {'$or': [{'_id': 1}]}";
@@ -200,7 +196,7 @@ test_dollar_or (void)
 static void
 test_snapshot_dollar_or (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.filter = "{'$or': [{'_id': 1}]}";
    test_data.opts = "{'snapshot': true}";
@@ -218,7 +214,7 @@ test_snapshot_dollar_or (void)
 static void
 test_key_named_filter (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.filter = "{'filter': 2}";
    test_data.expected_op_query = " {'filter': 2}";
@@ -232,7 +228,7 @@ test_key_named_filter (void)
 static void
 test_op_query_subdoc_named_filter (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.filter = "{'filter': {'i': 2}}";
    test_data.expected_op_query = " {'filter': {'i': 2}}";
@@ -249,7 +245,7 @@ test_op_query_subdoc_named_filter (void)
 static void
 test_find_cmd_subdoc_named_filter_with_option (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.filter = "{'filter': {'i': 2}}";
    test_data.opts = "{'snapshot': true}";
@@ -262,12 +258,11 @@ test_find_cmd_subdoc_named_filter_with_option (void)
    _test_collection_find_with_opts (&test_data);
 }
 
-/* this can't work until later in CDRIVER-1522 implementation */
-/* test future-compatibility with a new server's find command options */
+
 static void
 test_newoption (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.filter = "{'_id': 1}";
    test_data.opts = "{'newOption': true}";
@@ -282,7 +277,7 @@ test_newoption (void)
 static void
 test_sort (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'sort': {'_id': -1}}";
    test_data.expected_op_query = "{'$query': {}, '$orderby': {'_id': -1}}";
@@ -295,10 +290,10 @@ test_sort (void)
 static void
 test_fields (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'projection': {'_id': 0, 'b': 1}}";
-   test_data.expected_op_query_projection = "{'_id': 0, 'b': 1}}";
+   test_data.expected_op_query_projection = "{'_id': 0, 'b': 1}";
    test_data.expected_find_command =
       "{'find': 'collection', 'filter': {}, 'projection': {'_id': 0, 'b': 1}}";
    _test_collection_find_with_opts (&test_data);
@@ -308,14 +303,14 @@ test_fields (void)
 static void
 test_slice (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'projection': {'array': {'$slice': 10}}}";
    test_data.expected_op_query_projection = "{'array': {'$slice': 10}}";
    test_data.expected_find_command =
       "{'find': 'collection', "
-       " 'filter': {},"
-       " 'projection': {'array': {'$slice': 10}}}";
+      " 'filter': {},"
+      " 'projection': {'array': {'$slice': 10}}}";
    _test_collection_find_with_opts (&test_data);
 }
 
@@ -324,8 +319,7 @@ static void
 test_int_modifiers (void)
 {
    const char *modifiers[] = {
-      "maxScan",
-      "maxTimeMS",
+      "maxScan", "maxTimeMS",
    };
 
    const char *mod;
@@ -333,18 +327,19 @@ test_int_modifiers (void)
    char *opts;
    char *query;
    char *find_command;
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    for (i = 0; i < sizeof (modifiers) / sizeof (const char *); i++) {
       mod = modifiers[i];
       opts = bson_strdup_printf ("{'%s': {'$numberLong': '9999'}}", mod);
       query = bson_strdup_printf ("{'$query': {},"
-                                  " '$%s': {'$numberLong': '9999'}}", mod);
+                                  " '$%s': {'$numberLong': '9999'}}",
+                                  mod);
 
       /* find command has same modifier, without the $-prefix */
-      find_command = bson_strdup_printf (
-         "{'find': 'collection', 'filter': {},"
-         " '%s': {'$numberLong': '9999'}}", mod);
+      find_command = bson_strdup_printf ("{'find': 'collection', 'filter': {},"
+                                         " '%s': {'$numberLong': '9999'}}",
+                                         mod);
 
       test_data.opts = opts;
       test_data.expected_op_query = query;
@@ -362,9 +357,7 @@ static void
 test_index_spec_modifiers (void)
 {
    const char *modifiers[] = {
-      "hint",
-      "min",
-      "max",
+      "hint", "min", "max",
    };
 
    const char *mod;
@@ -372,7 +365,7 @@ test_index_spec_modifiers (void)
    char *opts;
    char *query;
    char *find_command;
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    for (i = 0; i < sizeof (modifiers) / sizeof (const char *); i++) {
       mod = modifiers[i];
@@ -396,33 +389,34 @@ test_index_spec_modifiers (void)
 }
 
 #undef COMMENT
-#define COMMENT "This is a long comment to ensure the opts and filter" \
+#define COMMENT                                                                \
+   "This is a long comment to ensure the opts and filter"                      \
    " documents overflow the bson_t inline storage, therefore revealing memory" \
-   " management bugs when we run the tests in valgrind." \
-   " Far over the misty mountains cold" \
-   " To dungeons deep and caverns old" \
-   " We must away ere break of day" \
-   " To seek the pale enchanted gold." \
-   " " \
-   " The dwarves of yore made mighty spells," \
-   " While hammers fell like ringing bells" \
-   " In places deep, where dark things sleep," \
-   " In hollow halls beneath the fells." \
-   " " \
-   " For ancient king and elvish lord" \
-   " There many a gleaming golden hoard" \
-   " They shaped and wrought, and light they caught" \
-   " To hide in gems on hilt of sword." \
-   " " \
-   " On silver necklaces they strung" \
-   " The flowering stars, on crowns they hung" \
-   " The dragon-fire, in twisted wire" \
+   " management bugs when we run the tests in valgrind."                       \
+   " Far over the misty mountains cold"                                        \
+   " To dungeons deep and caverns old"                                         \
+   " We must away ere break of day"                                            \
+   " To seek the pale enchanted gold."                                         \
+   " "                                                                         \
+   " The dwarves of yore made mighty spells,"                                  \
+   " While hammers fell like ringing bells"                                    \
+   " In places deep, where dark things sleep,"                                 \
+   " In hollow halls beneath the fells."                                       \
+   " "                                                                         \
+   " For ancient king and elvish lord"                                         \
+   " There many a gleaming golden hoard"                                       \
+   " They shaped and wrought, and light they caught"                           \
+   " To hide in gems on hilt of sword."                                        \
+   " "                                                                         \
+   " On silver necklaces they strung"                                          \
+   " The flowering stars, on crowns they hung"                                 \
+   " The dragon-fire, in twisted wire"                                         \
    " They meshed the light of moon and sun."
 
 static void
 test_comment (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'comment': '" COMMENT "'}";
    test_data.expected_op_query = "{'$query': {}, '$comment': '" COMMENT "'}";
@@ -436,7 +430,7 @@ test_comment (void)
 static void
 test_snapshot (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'snapshot': true}";
    test_data.expected_op_query = "{'$query': {}, '$snapshot': true}";
@@ -450,7 +444,7 @@ test_snapshot (void)
 static void
 test_diskloc (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'showRecordId': true}";
    test_data.expected_op_query = "{'$query': {}, '$showDiskLoc': true}";
@@ -463,7 +457,7 @@ test_diskloc (void)
 static void
 test_returnkey (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'returnKey': true}";
    test_data.expected_op_query = "{'$query': {}, '$returnKey': true}";
@@ -476,7 +470,7 @@ test_returnkey (void)
 static void
 test_skip (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.expected_skip = 1;
    test_data.opts = "{'skip': {'$numberLong': '1'}}";
@@ -489,7 +483,7 @@ test_skip (void)
 static void
 test_batch_size (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'batchSize': {'$numberLong': '2'}}";
    test_data.expected_n_return = 2;
@@ -502,7 +496,7 @@ test_batch_size (void)
 static void
 test_limit (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'limit': {'$numberLong': '2'}}";
    test_data.expected_n_return = 2;
@@ -515,7 +509,7 @@ test_limit (void)
 static void
 test_singlebatch (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'limit': {'$numberLong': '2'}, 'singleBatch': true}";
    test_data.expected_n_return = -2;
@@ -529,7 +523,7 @@ test_singlebatch (void)
 static void
 test_singlebatch_no_limit (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'singleBatch': true}";
    /* singleBatch doesn't affect OP_QUERY with limit 0, nToReturn is still 0 */
@@ -544,7 +538,7 @@ test_singlebatch_no_limit (void)
 static void
 test_unrecognized_dollar_option (void)
 {
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
    test_data.opts = "{'dumb': 1}";
    test_data.expected_op_query = "{'$query': {}, '$dumb': 1}";
@@ -561,43 +555,27 @@ test_query_flags (void)
    int i;
    char *opts;
    char *find_cmd;
-   test_collection_find_with_opts_t test_data = { 0 };
+   test_collection_find_with_opts_t test_data = {0};
 
-   typedef struct
-   {
-      mongoc_query_flags_t  flag;
-      const char           *json_fragment;
+   typedef struct {
+      mongoc_query_flags_t flag;
+      const char *json_fragment;
    } flag_and_name_t;
 
    /* slaveok is not supported as an option, exhaust is tested separately */
    flag_and_name_t flags_and_frags[] = {
-      {
-         MONGOC_QUERY_TAILABLE_CURSOR,
-         "'tailable': true"
-      },
-      {
-         MONGOC_QUERY_OPLOG_REPLAY,
-         "'oplogReplay': true"
-      },
-      {
-         MONGOC_QUERY_NO_CURSOR_TIMEOUT,
-         "'noCursorTimeout': true"
-      },
-      {
-         MONGOC_QUERY_PARTIAL,
-         "'allowPartialResults': true"
-      },
-      {
-         MONGOC_QUERY_TAILABLE_CURSOR | MONGOC_QUERY_AWAIT_DATA,
-         "'tailable': true, 'awaitData': true"
-      },
+      {MONGOC_QUERY_TAILABLE_CURSOR, "'tailable': true"},
+      {MONGOC_QUERY_OPLOG_REPLAY, "'oplogReplay': true"},
+      {MONGOC_QUERY_NO_CURSOR_TIMEOUT, "'noCursorTimeout': true"},
+      {MONGOC_QUERY_PARTIAL, "'allowPartialResults': true"},
+      {MONGOC_QUERY_TAILABLE_CURSOR | MONGOC_QUERY_AWAIT_DATA,
+       "'tailable': true, 'awaitData': true"},
    };
 
    for (i = 0; i < (sizeof flags_and_frags) / (sizeof (flag_and_name_t)); i++) {
       opts = bson_strdup_printf ("{%s}", flags_and_frags[i].json_fragment);
-      find_cmd = bson_strdup_printf (
-         "{'find': 'collection', 'filter': {}, %s}",
-         flags_and_frags[i].json_fragment);
+      find_cmd = bson_strdup_printf ("{'find': 'collection', 'filter': {}, %s}",
+                                     flags_and_frags[i].json_fragment);
 
       test_data.opts = opts;
       test_data.expected_flags = flags_and_frags[i].flag;
@@ -627,9 +605,8 @@ test_exhaust (void)
    mock_server_run (server);
    client = mongoc_client_new_from_uri (mock_server_get_uri (server));
    collection = mongoc_client_get_collection (client, "db", "collection");
-   cursor = mongoc_collection_find_with_opts (collection, tmp_bson (NULL),
-                                              tmp_bson ("{'exhaust': true}"),
-                                              NULL);
+   cursor = mongoc_collection_find_with_opts (
+      collection, tmp_bson (NULL), tmp_bson ("{'exhaust': true}"), NULL);
 
    future = future_cursor_next (cursor, &doc);
 
@@ -638,9 +615,13 @@ test_exhaust (void)
     * fallback to existing OP_QUERY wire protocol messages."
     */
    request = mock_server_receives_request (server);
-   mock_server_replies_to_find (
-      request, MONGOC_QUERY_SLAVE_OK | MONGOC_QUERY_EXHAUST,
-      0, 0, "db.collection", "{}", false /* is_command */);
+   mock_server_replies_to_find (request,
+                                MONGOC_QUERY_SLAVE_OK | MONGOC_QUERY_EXHAUST,
+                                0,
+                                0,
+                                "db.collection",
+                                "{}",
+                                false /* is_command */);
 
    ASSERT (future_get_bool (future));
    ASSERT_OR_PRINT (!mongoc_cursor_error (cursor, &error), error);
@@ -677,23 +658,26 @@ test_getmore_cmd_await (void)
    mock_server_run (server);
    client = mongoc_client_new_from_uri (mock_server_get_uri (server));
    collection = mongoc_client_get_collection (client, "db", "collection");
-   cursor = mongoc_collection_find_with_opts (collection, tmp_bson (NULL), opts,
-                                              NULL);
+   cursor = mongoc_collection_find_with_opts (
+      collection, tmp_bson (NULL), opts, NULL);
 
    future = future_cursor_next (cursor, &doc);
-   request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_SLAVE_OK,
-      "{'find': 'collection',"
-      " 'filter': {},"
-      " 'maxTimeMS': {'$exists': false},"
-      " 'maxAwaitTimeMS': {'$exists': false}}");
+   request =
+      mock_server_receives_command (server,
+                                    "db",
+                                    MONGOC_QUERY_SLAVE_OK,
+                                    "{'find': 'collection',"
+                                    " 'filter': {},"
+                                    " 'maxTimeMS': {'$exists': false},"
+                                    " 'maxAwaitTimeMS': {'$exists': false}}");
 
    ASSERT (request);
-   mock_server_replies_simple (request, "{'ok': 1,"
-                                        " 'cursor': {"
-                                        "    'id': {'$numberLong': '123'},"
-                                        "    'ns': 'db.collection',"
-                                        "    'firstBatch': [{}]}}");
+   mock_server_replies_simple (request,
+                               "{'ok': 1,"
+                               " 'cursor': {"
+                               "    'id': {'$numberLong': '123'},"
+                               "    'ns': 'db.collection',"
+                               "    'firstBatch': [{}]}}");
 
    ASSERT (future_get_bool (future));
    request_destroy (request);
@@ -703,19 +687,22 @@ test_getmore_cmd_await (void)
     * "getMore" command
     */
    future = future_cursor_next (cursor, &doc);
-   request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_SLAVE_OK,
-      "{'getMore': {'$numberLong': '123'},"
-      " 'collection': 'collection',"
-      " 'maxAwaitTimeMS': {'$exists': false},"
-      " 'maxTimeMS': {'$numberLong': '9999'}}}");
+   request =
+      mock_server_receives_command (server,
+                                    "db",
+                                    MONGOC_QUERY_SLAVE_OK,
+                                    "{'getMore': {'$numberLong': '123'},"
+                                    " 'collection': 'collection',"
+                                    " 'maxAwaitTimeMS': {'$exists': false},"
+                                    " 'maxTimeMS': {'$numberLong': '9999'}}");
 
    ASSERT (request);
-   mock_server_replies_simple (request, "{'ok': 1,"
-                                        " 'cursor': {"
-                                        "    'id': {'$numberLong': '0'},"
-                                        "    'ns': 'db.collection',"
-                                        "    'nextBatch': [{}]}}");
+   mock_server_replies_simple (request,
+                               "{'ok': 1,"
+                               " 'cursor': {"
+                               "    'id': {'$numberLong': '0'},"
+                               "    'ns': 'db.collection',"
+                               "    'nextBatch': [{}]}}");
 
    ASSERT (future_get_bool (future));
 
@@ -740,10 +727,10 @@ test_find_w_server_id (void)
    future_t *future;
    request_t *request;
 
-   rs = mock_rs_with_autoismaster (0    /* wire version */,
+   rs = mock_rs_with_autoismaster (WIRE_VERSION_MIN /* wire version */,
                                    true /* has primary  */,
-                                   1    /* secondary    */,
-                                   0    /* arbiters     */);
+                                   1 /* secondary    */,
+                                   0 /* arbiters     */);
 
    mock_rs_run (rs);
    client = mongoc_client_new_from_uri (mock_rs_get_uri (rs));
@@ -751,12 +738,12 @@ test_find_w_server_id (void)
 
    /* use serverId instead of prefs to select the secondary */
    opts = tmp_bson ("{'serverId': 2}");
-   cursor = mongoc_collection_find_with_opts (collection, tmp_bson (NULL), opts,
-                                              NULL);
+   cursor = mongoc_collection_find_with_opts (
+      collection, tmp_bson (NULL), opts, NULL);
 
    future = future_cursor_next (cursor, &doc);
-   request = mock_rs_receives_query (rs, "db.collection", MONGOC_QUERY_SLAVE_OK,
-                                     0, 0, "{}", NULL);
+   request = mock_rs_receives_query (
+      rs, "db.collection", MONGOC_QUERY_SLAVE_OK, 0, 0, "{}", NULL);
 
    ASSERT (mock_rs_request_is_to_secondary (rs, request));
    mock_rs_replies_simple (request, "{}");
@@ -786,8 +773,8 @@ test_find_cmd_w_server_id (void)
 
    rs = mock_rs_with_autoismaster (WIRE_VERSION_READ_CONCERN,
                                    true /* has primary  */,
-                                   1    /* secondary    */,
-                                   0    /* arbiters     */);
+                                   1 /* secondary    */,
+                                   0 /* arbiters     */);
 
    mock_rs_run (rs);
    client = mongoc_client_new_from_uri (mock_rs_get_uri (rs));
@@ -795,25 +782,27 @@ test_find_cmd_w_server_id (void)
 
    /* use serverId instead of prefs to select the secondary */
    opts = tmp_bson ("{'serverId': 2, 'readConcern': {'level': 'local'}}");
-   cursor = mongoc_collection_find_with_opts (collection, tmp_bson (NULL), opts,
-                                              NULL);
+   cursor = mongoc_collection_find_with_opts (
+      collection, tmp_bson (NULL), opts, NULL);
 
    future = future_cursor_next (cursor, &doc);
 
    /* recognized that wire version is recent enough for readConcern */
-   request = mock_rs_receives_command (
-      rs, "db", MONGOC_QUERY_SLAVE_OK,
-      "{'find': 'collection', "
-      " 'filter': {},"
-      " 'readConcern': {'level': 'local'},"
-      " 'serverId': {'$exists': false}}");
+   request = mock_rs_receives_command (rs,
+                                       "db",
+                                       MONGOC_QUERY_SLAVE_OK,
+                                       "{'find': 'collection', "
+                                       " 'filter': {},"
+                                       " 'readConcern': {'level': 'local'},"
+                                       " 'serverId': {'$exists': false}}");
 
    ASSERT (mock_rs_request_is_to_secondary (rs, request));
-   mock_rs_replies_simple (request, "{'ok': 1,"
-                                    " 'cursor': {"
-                                    "    'id': 0,"
-                                    "    'ns': 'db.collection',"
-                                    "    'firstBatch': [{}]}}");
+   mock_rs_replies_simple (request,
+                           "{'ok': 1,"
+                           " 'cursor': {"
+                           "    'id': 0,"
+                           "    'ns': 'db.collection',"
+                           "    'firstBatch': [{}]}}");
 
    ASSERT_OR_PRINT (future_get_bool (future), error);
 
@@ -839,21 +828,20 @@ test_find_w_server_id_sharded (void)
    request_t *request;
    bson_error_t error;
 
-   server = mock_mongos_new (0);
+   server = mock_mongos_new (WIRE_VERSION_MIN);
    mock_server_run (server);
    client = mongoc_client_new_from_uri (mock_server_get_uri (server));
    collection = mongoc_client_get_collection (client, "db", "collection");
 
    opts = tmp_bson ("{'serverId': 1}");
-   cursor = mongoc_collection_find_with_opts (collection, tmp_bson (NULL), opts,
-                                              NULL);
+   cursor = mongoc_collection_find_with_opts (
+      collection, tmp_bson (NULL), opts, NULL);
 
    future = future_cursor_next (cursor, &doc);
 
    /* does NOT set slave ok, since this is a sharded topology */
-   request = mock_server_receives_query (server, "db.collection",
-                                         MONGOC_QUERY_NONE, 0, 0, "{}",
-                                         NULL);
+   request = mock_server_receives_query (
+      server, "db.collection", MONGOC_QUERY_NONE, 0, 0, "{}", NULL);
 
    mock_server_replies_simple (request, "{}");
    ASSERT_OR_PRINT (future_get_bool (future), error);
@@ -886,25 +874,27 @@ test_find_cmd_w_server_id_sharded (void)
    collection = mongoc_client_get_collection (client, "db", "collection");
 
    opts = tmp_bson ("{'serverId': 1, 'readConcern': {'level': 'local'}}");
-   cursor = mongoc_collection_find_with_opts (collection, tmp_bson (NULL), opts,
-                                              NULL);
+   cursor = mongoc_collection_find_with_opts (
+      collection, tmp_bson (NULL), opts, NULL);
 
    future = future_cursor_next (cursor, &doc);
 
    /* recognized that wire version is recent enough for readConcern */
    /* does NOT set slave ok, since this is a sharded topology */
-   request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_NONE,
-      "{'find': 'collection', "
-      " 'filter': {},"
-      " 'readConcern': {'level': 'local'},"
-      " 'serverId': {'$exists': false}}");
+   request = mock_server_receives_command (server,
+                                           "db",
+                                           MONGOC_QUERY_NONE,
+                                           "{'find': 'collection', "
+                                           " 'filter': {},"
+                                           " 'readConcern': {'level': 'local'},"
+                                           " 'serverId': {'$exists': false}}");
 
-   mock_rs_replies_simple (request, "{'ok': 1,"
-                                    " 'cursor': {"
-                                    "    'id': 0,"
-                                    "    'ns': 'db.collection',"
-                                    "    'firstBatch': [{}]}}");
+   mock_rs_replies_simple (request,
+                           "{'ok': 1,"
+                           " 'cursor': {"
+                           "    'id': 0,"
+                           "    'ns': 'db.collection',"
+                           "    'firstBatch': [{}]}}");
 
    ASSERT_OR_PRINT (future_get_bool (future), error);
 
@@ -931,7 +921,8 @@ test_server_id_option (void)
    cursor = mongoc_collection_find_with_opts (
       collection, q, tmp_bson ("{'serverId': 'foo'}"), NULL);
 
-   ASSERT_ERROR_CONTAINS (cursor->error, MONGOC_ERROR_CURSOR,
+   ASSERT_ERROR_CONTAINS (cursor->error,
+                          MONGOC_ERROR_CURSOR,
                           MONGOC_ERROR_CURSOR_INVALID_CURSOR,
                           "must be an integer");
 
@@ -939,14 +930,14 @@ test_server_id_option (void)
    cursor = mongoc_collection_find_with_opts (
       collection, q, tmp_bson ("{'serverId': 0}"), NULL);
 
-   ASSERT_ERROR_CONTAINS (cursor->error, MONGOC_ERROR_CURSOR,
+   ASSERT_ERROR_CONTAINS (cursor->error,
+                          MONGOC_ERROR_CURSOR,
                           MONGOC_ERROR_CURSOR_INVALID_CURSOR,
                           "must be >= 1");
 
    mongoc_cursor_destroy (cursor);
    cursor = mongoc_collection_find_with_opts (
-      collection, q, tmp_bson ("{'serverId': 1}"),
-      NULL);
+      collection, q, tmp_bson ("{'serverId': 1}"), NULL);
 
    ASSERT_OR_PRINT (!mongoc_cursor_error (cursor, &error), error);
 
@@ -976,10 +967,11 @@ test_find_with_opts_collation_error (void *ctx)
       ASSERT (false);
    }
 
-   ASSERT ( mongoc_cursor_error (cursor, &error));
-   ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_CURSOR,
+   ASSERT (mongoc_cursor_error (cursor, &error));
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_COMMAND,
                           MONGOC_ERROR_PROTOCOL_BAD_WIRE_VERSION,
-                          "Collation is not supported by this server");
+                          "The selected server does not support collation");
 
    mongoc_cursor_destroy (cursor);
    mongoc_collection_destroy (collection);
@@ -989,66 +981,87 @@ test_find_with_opts_collation_error (void *ctx)
 void
 test_collection_find_with_opts_install (TestSuite *suite)
 {
-   TestSuite_Add (suite, "/Collection/find_with_opts/dollar_or",
-                  test_dollar_or);
-   TestSuite_Add (suite, "/Collection/find_with_opts/snapshot_dollar_or",
-                  test_snapshot_dollar_or);
-   TestSuite_Add (suite, "/Collection/find_with_opts/key_named_filter",
-                  test_key_named_filter);
-   TestSuite_Add (suite,
-                  "/Collection/find_with_opts/query/subdoc_named_filter",
-                  test_op_query_subdoc_named_filter);
-   TestSuite_Add (suite, "/Collection/find_with_opts/newoption",
-                  test_newoption);
-   TestSuite_Add (suite, "/Collection/find_with_opts/cmd/subdoc_named_filter",
-                  test_find_cmd_subdoc_named_filter_with_option);
-   TestSuite_Add (suite, "/Collection/find_with_opts/orderby",
-                  test_sort);
-   TestSuite_Add (suite, "/Collection/find_with_opts/fields",
-                  test_fields);
-   TestSuite_Add (suite, "/Collection/find_with_opts/slice",
-                  test_slice);
-   TestSuite_Add (suite, "/Collection/find_with_opts/modifiers/integer",
-                  test_int_modifiers);
-   TestSuite_Add (suite, "/Collection/find_with_opts/modifiers/index_spec",
-                  test_index_spec_modifiers);
-   TestSuite_Add (suite, "/Collection/find_with_opts/comment",
-                  test_comment);
-   TestSuite_Add (suite, "/Collection/find_with_opts/modifiers/bool",
-                  test_snapshot);
-   TestSuite_Add (suite, "/Collection/find_with_opts/showdiskloc",
-                  test_diskloc);
-   TestSuite_Add (suite, "/Collection/find_with_opts/returnkey",
-                  test_returnkey);
-   TestSuite_Add (suite, "/Collection/find_with_opts/skip",
-                  test_skip);
-   TestSuite_Add (suite, "/Collection/find_with_opts/batch_size",
-                  test_batch_size);
-   TestSuite_Add (suite, "/Collection/find_with_opts/limit",
-                  test_limit);
-   TestSuite_Add (suite, "/Collection/find_with_opts/singlebatch",
-                  test_singlebatch);
-   TestSuite_Add (suite, "/Collection/find_with_opts/singlebatch/no_limit",
-                  test_singlebatch_no_limit);
-   TestSuite_Add (suite, "/Collection/find_with_opts/unrecognized_dollar",
-                  test_unrecognized_dollar_option);
-   TestSuite_Add (suite, "/Collection/find_with_opts/flags",
-                  test_query_flags);
-   TestSuite_Add (suite, "/Collection/find_with_opts/exhaust",
-                  test_exhaust);
-   TestSuite_Add (suite, "/Collection/find_with_opts/await/getmore_cmd",
-                  test_getmore_cmd_await);
-   TestSuite_Add (suite, "/Collection/find_with_opts/server_id",
-                  test_find_w_server_id);
-   TestSuite_Add (suite, "/Collection/find_cmd_with_opts/server_id",
-                  test_find_cmd_w_server_id);
-   TestSuite_Add (suite, "/Collection/find_with_opts/server_id/sharded",
-                  test_find_w_server_id_sharded);
-   TestSuite_Add (suite, "/Collection/find_cmd_with_opts/server_id/sharded",
-                  test_find_cmd_w_server_id_sharded);
-   TestSuite_AddLive (suite, "/Collection/find_with_opts/server_id/option",
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/dollar_or", test_dollar_or);
+   TestSuite_AddMockServerTest (suite,
+                                "/Collection/find_with_opts/snapshot_dollar_or",
+                                test_snapshot_dollar_or);
+   TestSuite_AddMockServerTest (suite,
+                                "/Collection/find_with_opts/key_named_filter",
+                                test_key_named_filter);
+   TestSuite_AddMockServerTest (
+      suite,
+      "/Collection/find_with_opts/query/subdoc_named_filter",
+      test_op_query_subdoc_named_filter);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/newoption", test_newoption);
+   TestSuite_AddMockServerTest (
+      suite,
+      "/Collection/find_with_opts/cmd/subdoc_named_filter",
+      test_find_cmd_subdoc_named_filter_with_option);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/orderby", test_sort);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/fields", test_fields);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/slice", test_slice);
+   TestSuite_AddMockServerTest (suite,
+                                "/Collection/find_with_opts/modifiers/integer",
+                                test_int_modifiers);
+   TestSuite_AddMockServerTest (
+      suite,
+      "/Collection/find_with_opts/modifiers/index_spec",
+      test_index_spec_modifiers);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/comment", test_comment);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/modifiers/bool", test_snapshot);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/showdiskloc", test_diskloc);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/returnkey", test_returnkey);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/skip", test_skip);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/batch_size", test_batch_size);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/limit", test_limit);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/singlebatch", test_singlebatch);
+   TestSuite_AddMockServerTest (
+      suite,
+      "/Collection/find_with_opts/singlebatch/no_limit",
+      test_singlebatch_no_limit);
+   TestSuite_AddMockServerTest (
+      suite,
+      "/Collection/find_with_opts/unrecognized_dollar",
+      test_unrecognized_dollar_option);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/flags", test_query_flags);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/exhaust", test_exhaust);
+   TestSuite_AddMockServerTest (suite,
+                                "/Collection/find_with_opts/await/getmore_cmd",
+                                test_getmore_cmd_await);
+   TestSuite_AddMockServerTest (
+      suite, "/Collection/find_with_opts/server_id", test_find_w_server_id);
+   TestSuite_AddMockServerTest (suite,
+                                "/Collection/find_cmd_with_opts/server_id",
+                                test_find_cmd_w_server_id);
+   TestSuite_AddMockServerTest (suite,
+                                "/Collection/find_with_opts/server_id/sharded",
+                                test_find_w_server_id_sharded);
+   TestSuite_AddMockServerTest (
+      suite,
+      "/Collection/find_cmd_with_opts/server_id/sharded",
+      test_find_cmd_w_server_id_sharded);
+   TestSuite_AddLive (suite,
+                      "/Collection/find_with_opts/server_id/option",
                       test_server_id_option);
-   TestSuite_AddFull (suite, "/Collection/find_with_opts/collation/error",
-                      test_find_with_opts_collation_error, NULL, NULL,
+   TestSuite_AddFull (suite,
+                      "/Collection/find_with_opts/collation/error",
+                      test_find_with_opts_collation_error,
+                      NULL,
+                      NULL,
                       test_framework_skip_if_max_wire_version_more_than_4);
 }
