@@ -2,6 +2,7 @@
 #include <mongoc-collection-private.h>
 #include <mongoc-write-concern-private.h>
 #include <mongoc-read-concern-private.h>
+#include <mongoc-util-private.h>
 
 #include "TestSuite.h"
 #include "test-libmongoc.h"
@@ -18,7 +19,7 @@ test_create_with_write_concern (void)
    mongoc_database_t *database;
    mongoc_collection_t *collection;
    mongoc_client_t *client;
-   bson_error_t error = { 0 };
+   bson_error_t error = {0};
    mongoc_write_concern_t *bad_wc;
    mongoc_write_concern_t *good_wc;
    bool wire_version_5;
@@ -30,7 +31,7 @@ test_create_with_write_concern (void)
    opts = bson_new ();
 
    client = test_framework_client_new ();
-   assert (client);
+   BSON_ASSERT (client);
    mongoc_client_set_error_api (client, 2);
 
    bad_wc = mongoc_write_concern_new ();
@@ -40,7 +41,7 @@ test_create_with_write_concern (void)
 
    dbname = gen_collection_name ("dbtest");
    database = mongoc_client_get_database (client, dbname);
-   assert (database);
+   BSON_ASSERT (database);
 
    name = gen_collection_name ("create_collection");
 
@@ -48,9 +49,10 @@ test_create_with_write_concern (void)
    bad_wc->wtimeout = -10;
    bson_reinit (opts);
    mongoc_write_concern_append_bad (bad_wc, opts);
-   collection = mongoc_database_create_collection (
-      database, name, opts, &error);
-   ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_COMMAND,
+   collection =
+      mongoc_database_create_collection (database, name, opts, &error);
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_COMMAND,
                           MONGOC_ERROR_COMMAND_INVALID_ARG,
                           "Invalid writeConcern");
    ASSERT (!collection);
@@ -62,14 +64,14 @@ test_create_with_write_concern (void)
    mongoc_write_concern_set_w (good_wc, 1);
    bson_reinit (opts);
    mongoc_write_concern_append (good_wc, opts);
-   collection = mongoc_database_create_collection (
-      database, name, opts, &error);
+   collection =
+      mongoc_database_create_collection (database, name, opts, &error);
    ASSERT_OR_PRINT (collection, error);
    ASSERT (!error.code);
    ASSERT (!error.domain);
 
-   ASSERT_OR_PRINT (mongoc_collection_drop (collection, &error),
-                    error);
+   ASSERT_OR_PRINT (mongoc_collection_drop (collection, &error), error);
+   mongoc_collection_destroy (collection);
 
    /* writeConcern that results in writeConcernError */
    bad_wc->wtimeout = 0;
@@ -77,14 +79,14 @@ test_create_with_write_concern (void)
    if (!test_framework_is_mongos ()) { /* skip if sharded */
       bson_reinit (opts);
       mongoc_write_concern_append_bad (bad_wc, opts);
-      collection = mongoc_database_create_collection (
-         database, name, opts, &error);
+      collection =
+         mongoc_database_create_collection (database, name, opts, &error);
 
       if (wire_version_5) {
          ASSERT (!collection);
          if (test_framework_is_replset ()) { /* replica set */
-            ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_WRITE_CONCERN,
-                                   100, "Write Concern error:");
+            ASSERT_ERROR_CONTAINS (
+               error, MONGOC_ERROR_WRITE_CONCERN, 100, "Write Concern error:");
          } else { /* standalone */
             ASSERT_CMPINT (error.domain, ==, MONGOC_ERROR_SERVER);
             ASSERT_CMPINT (error.code, ==, 2);
@@ -93,8 +95,8 @@ test_create_with_write_concern (void)
          ASSERT_OR_PRINT (collection, error);
          ASSERT (!error.code);
          ASSERT (!error.domain);
-         ASSERT_OR_PRINT (mongoc_collection_drop (collection, &error),
-                          error);
+         ASSERT_OR_PRINT (mongoc_collection_drop (collection, &error), error);
+         mongoc_collection_destroy (collection);
       }
    }
 
@@ -121,14 +123,14 @@ test_copy (void)
    database = mongoc_client_get_database (client, "test");
    ASSERT (database);
 
-   copy = mongoc_database_copy(database);
+   copy = mongoc_database_copy (database);
    ASSERT (copy);
    ASSERT (copy->client == database->client);
-   ASSERT (strcmp(copy->name, database->name) == 0);
+   ASSERT (strcmp (copy->name, database->name) == 0);
 
-   mongoc_database_destroy(copy);
-   mongoc_database_destroy(database);
-   mongoc_client_destroy(client);
+   mongoc_database_destroy (copy);
+   mongoc_database_destroy (database);
+   mongoc_client_destroy (client);
 }
 
 static void
@@ -144,27 +146,26 @@ test_has_collection (void)
    bson_t b;
 
    client = test_framework_client_new ();
-   assert (client);
+   BSON_ASSERT (client);
 
    name = gen_collection_name ("has_collection");
    collection = mongoc_client_get_collection (client, "test", name);
-   assert (collection);
+   BSON_ASSERT (collection);
 
    database = mongoc_client_get_database (client, "test");
-   assert (database);
+   BSON_ASSERT (database);
 
    bson_init (&b);
    bson_oid_init (&oid, NULL);
    bson_append_oid (&b, "_id", 3, &oid);
    bson_append_utf8 (&b, "hello", 5, "world", 5);
-   ASSERT_OR_PRINT (mongoc_collection_insert (collection, MONGOC_INSERT_NONE,
-                                              &b, NULL, &error),
-                    error);
+   ASSERT_OR_PRINT (
+      mongoc_collection_insert_one (collection, &b, NULL, NULL, &error), error);
    bson_destroy (&b);
 
    r = mongoc_database_has_collection (database, name, &error);
-   assert (!error.domain);
-   assert (r);
+   BSON_ASSERT (!error.domain);
+   BSON_ASSERT (r);
 
    bson_free (name);
    mongoc_database_destroy (database);
@@ -186,7 +187,7 @@ test_command (void)
    bson_t reply;
 
    client = test_framework_client_new ();
-   assert (client);
+   BSON_ASSERT (client);
 
    database = mongoc_client_get_database (client, "admin");
 
@@ -195,16 +196,17 @@ test_command (void)
     */
    bson_append_int32 (&cmd, "ping", 4, 1);
 
-   cursor = mongoc_database_command (database, MONGOC_QUERY_NONE, 0, 1, 0, &cmd, NULL, NULL);
-   assert (cursor);
+   cursor = mongoc_database_command (
+      database, MONGOC_QUERY_NONE, 0, 1, 0, &cmd, NULL, NULL);
+   BSON_ASSERT (cursor);
 
    r = mongoc_cursor_next (cursor, &doc);
-   assert (r);
-   assert (doc);
+   BSON_ASSERT (r);
+   BSON_ASSERT (doc);
 
    r = mongoc_cursor_next (cursor, &doc);
-   assert (!r);
-   assert (!doc);
+   BSON_ASSERT (!r);
+   BSON_ASSERT (!doc);
 
    mongoc_cursor_destroy (cursor);
 
@@ -216,10 +218,10 @@ test_command (void)
    bson_append_int32 (&cmd, "a_non_existing_command", -1, 1);
 
    r = mongoc_database_command_simple (database, &cmd, NULL, &reply, &error);
-   assert (!r);
-   assert (error.domain == MONGOC_ERROR_QUERY);
-   assert (error.code == MONGOC_ERROR_QUERY_COMMAND_NOT_FOUND);
-   assert (strstr (error.message, "a_non_existing_command"));
+   BSON_ASSERT (!r);
+   BSON_ASSERT (error.domain == MONGOC_ERROR_QUERY);
+   BSON_ASSERT (error.code == MONGOC_ERROR_QUERY_COMMAND_NOT_FOUND);
+   BSON_ASSERT (strstr (error.message, "a_non_existing_command"));
 
    bson_destroy (&reply);
    mongoc_database_destroy (database);
@@ -244,7 +246,7 @@ _test_db_command_read_prefs (bool simple, bool pooled)
    const bson_t *reply;
 
    /* mock mongos: easiest way to test that read preference is configured */
-   server = mock_mongos_new (0);
+   server = mock_mongos_new (WIRE_VERSION_MIN);
    mock_server_run (server);
 
    if (pooled) {
@@ -261,8 +263,7 @@ _test_db_command_read_prefs (bool simple, bool pooled)
 
    if (simple) {
       /* simple, without read preference */
-      future = future_database_command_simple (db, cmd,
-                                               NULL, NULL, &error);
+      future = future_database_command_simple (db, cmd, NULL, NULL, &error);
 
       request = mock_server_receives_command (
          server, "db", MONGOC_QUERY_NONE, "{'foo': 1}");
@@ -273,11 +274,13 @@ _test_db_command_read_prefs (bool simple, bool pooled)
       request_destroy (request);
 
       /* with read preference */
-      future = future_database_command_simple (db, cmd,
-                                               secondary_pref, NULL, &error);
+      future =
+         future_database_command_simple (db, cmd, secondary_pref, NULL, &error);
 
       request = mock_server_receives_command (
-         server, "db", MONGOC_QUERY_SLAVE_OK,
+         server,
+         "db",
+         MONGOC_QUERY_SLAVE_OK,
          "{'$query': {'foo': 1},"
          " '$readPreference': {'mode': 'secondary'}}");
       mock_server_replies_simple (request, "{'ok': 1}");
@@ -286,8 +289,8 @@ _test_db_command_read_prefs (bool simple, bool pooled)
       request_destroy (request);
    } else {
       /* not simple, no read preference */
-      cursor = mongoc_database_command (db, MONGOC_QUERY_NONE, 0, 0, 0,
-                                        cmd, NULL, NULL);
+      cursor = mongoc_database_command (
+         db, MONGOC_QUERY_NONE, 0, 0, 0, cmd, NULL, NULL);
       future = future_cursor_next (cursor, &reply);
       request = mock_server_receives_command (
          server, "db", MONGOC_QUERY_NONE, "{'foo': 1}");
@@ -299,11 +302,13 @@ _test_db_command_read_prefs (bool simple, bool pooled)
       mongoc_cursor_destroy (cursor);
 
       /* with read preference */
-      cursor = mongoc_database_command (db, MONGOC_QUERY_NONE, 0, 0, 0,
-                                        cmd, NULL, secondary_pref);
+      cursor = mongoc_database_command (
+         db, MONGOC_QUERY_NONE, 0, 0, 0, cmd, NULL, secondary_pref);
       future = future_cursor_next (cursor, &reply);
       request = mock_server_receives_command (
-         server, "db", MONGOC_QUERY_SLAVE_OK,
+         server,
+         "db",
+         MONGOC_QUERY_SLAVE_OK,
          "{'$query': {'foo': 1},"
          " '$readPreference': {'mode': 'secondary'}}");
 
@@ -362,7 +367,7 @@ test_drop (void)
    mongoc_client_t *client;
    mongoc_database_t *database;
    mongoc_collection_t *collection;
-   bson_error_t error = { 0 };
+   bson_error_t error = {0};
    bson_t *opts = NULL;
    char *dbname;
    mongoc_write_concern_t *good_wc;
@@ -372,7 +377,7 @@ test_drop (void)
 
    opts = bson_new ();
    client = test_framework_client_new ();
-   assert (client);
+   BSON_ASSERT (client);
    mongoc_client_set_error_api (client, 2);
 
    bad_wc = mongoc_write_concern_new ();
@@ -385,14 +390,14 @@ test_drop (void)
    /* MongoDB 3.2+ must create at least one replicated database before
     * dropDatabase will check writeConcern, see SERVER-25601 */
    collection = mongoc_database_get_collection (database, "collection");
-   r = mongoc_collection_insert (collection, MONGOC_INSERT_NONE, tmp_bson ("{}"),
-                                 NULL, &error);
+   r = mongoc_collection_insert_one (
+      collection, tmp_bson ("{}"), NULL, NULL, &error);
 
    ASSERT_OR_PRINT (r, error);
 
    ASSERT_OR_PRINT (mongoc_database_drop (database, &error), error);
-   assert (!error.domain);
-   assert (!error.code);
+   BSON_ASSERT (!error.domain);
+   BSON_ASSERT (!error.code);
 
    mongoc_database_destroy (database);
 
@@ -402,10 +407,9 @@ test_drop (void)
 
    bson_reinit (opts);
    mongoc_write_concern_append_bad (bad_wc, opts);
-   ASSERT (!mongoc_database_drop_with_opts (database,
-                                            opts,
-                                            &error));
-   ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_COMMAND,
+   ASSERT (!mongoc_database_drop_with_opts (database, opts, &error));
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_COMMAND,
                           MONGOC_ERROR_COMMAND_INVALID_ARG,
                           "Invalid writeConcern");
    bad_wc->wtimeout = 0;
@@ -417,12 +421,10 @@ test_drop (void)
 
    bson_reinit (opts);
    mongoc_write_concern_append (good_wc, opts);
-   ASSERT_OR_PRINT (mongoc_database_drop_with_opts (database,
-                                                    opts,
-                                                    &error),
+   ASSERT_OR_PRINT (mongoc_database_drop_with_opts (database, opts, &error),
                     error);
-   assert (!error.code);
-   assert (!error.domain);
+   BSON_ASSERT (!error.code);
+   BSON_ASSERT (!error.domain);
 
    /* invalid writeConcern */
    mongoc_write_concern_set_w (bad_wc, 99);
@@ -432,14 +434,12 @@ test_drop (void)
       database = mongoc_client_get_database (client, dbname);
       bson_reinit (opts);
       mongoc_write_concern_append_bad (bad_wc, opts);
-      r = mongoc_database_drop_with_opts (database,
-                                          opts,
-                                          &error);
+      r = mongoc_database_drop_with_opts (database, opts, &error);
       if (wire_version_5) {
          ASSERT (!r);
          if (test_framework_is_replset ()) {
-            ASSERT_ERROR_CONTAINS (error, MONGOC_ERROR_WRITE_CONCERN,
-                                   100, "Write Concern error:");
+            ASSERT_ERROR_CONTAINS (
+               error, MONGOC_ERROR_WRITE_CONCERN, 100, "Write Concern error:");
          } else { /* standalone */
             ASSERT_CMPINT (error.domain, ==, MONGOC_ERROR_SERVER);
             ASSERT_CMPINT (error.code, ==, 2);
@@ -448,8 +448,8 @@ test_drop (void)
          ASSERT_OR_PRINT (r, error);
          ASSERT (!error.code);
          ASSERT (!error.domain);
-         mongoc_database_destroy (database);
       }
+      mongoc_database_destroy (database);
    }
 
    bson_free (dbname);
@@ -467,7 +467,7 @@ test_create_collection (void)
    mongoc_database_t *database;
    mongoc_collection_t *collection;
    mongoc_client_t *client;
-   bson_error_t error = { 0 };
+   bson_error_t error = {0};
    bson_t options;
    bson_t storage_opts;
    bson_t wt_opts;
@@ -476,11 +476,11 @@ test_create_collection (void)
    char *name;
 
    client = test_framework_client_new ();
-   assert (client);
+   BSON_ASSERT (client);
 
    dbname = gen_collection_name ("dbtest");
    database = mongoc_client_get_database (client, dbname);
-   assert (database);
+   BSON_ASSERT (database);
    bson_free (dbname);
 
    bson_init (&options);
@@ -488,18 +488,17 @@ test_create_collection (void)
    BSON_APPEND_INT32 (&options, "max", 4567);
    BSON_APPEND_BOOL (&options, "capped", true);
 
-   BSON_APPEND_DOCUMENT_BEGIN(&options, "storageEngine", &storage_opts);
-   BSON_APPEND_DOCUMENT_BEGIN(&storage_opts, "wiredTiger", &wt_opts);
-   BSON_APPEND_UTF8(&wt_opts, "configString", "block_compressor=zlib");
-   bson_append_document_end(&storage_opts, &wt_opts);
-   bson_append_document_end(&options, &storage_opts);
+   BSON_APPEND_DOCUMENT_BEGIN (&options, "storageEngine", &storage_opts);
+   BSON_APPEND_DOCUMENT_BEGIN (&storage_opts, "wiredTiger", &wt_opts);
+   BSON_APPEND_UTF8 (&wt_opts, "configString", "block_compressor=zlib");
+   bson_append_document_end (&storage_opts, &wt_opts);
+   bson_append_document_end (&options, &storage_opts);
 
 
    name = gen_collection_name ("create_collection");
-   ASSERT_OR_PRINT (
-      collection = mongoc_database_create_collection (database, name,
-                                                      &options, &error),
-      error);
+   ASSERT_OR_PRINT (collection = mongoc_database_create_collection (
+                       database, name, &options, &error),
+                    error);
 
    bson_destroy (&options);
    bson_free (name);
@@ -520,7 +519,7 @@ test_get_collection_info (void)
    mongoc_collection_t *collection;
    mongoc_client_t *client;
    mongoc_cursor_t *cursor;
-   bson_error_t error = { 0 };
+   bson_error_t error = {0};
    bson_iter_t col_iter;
    bson_t capped_options = BSON_INITIALIZER;
    bson_t autoindexid_options = BSON_INITIALIZER;
@@ -536,12 +535,12 @@ test_get_collection_info (void)
    char *noopts_name;
 
    client = test_framework_client_new ();
-   assert (client);
+   BSON_ASSERT (client);
 
    dbname = gen_collection_name ("dbtest");
    database = mongoc_client_get_database (client, dbname);
 
-   assert (database);
+   BSON_ASSERT (database);
    bson_free (dbname);
 
    capped_name = gen_collection_name ("capped");
@@ -553,19 +552,18 @@ test_get_collection_info (void)
 
    noopts_name = gen_collection_name ("noopts");
 
-   collection = mongoc_database_create_collection (database, capped_name,
-                                                   &capped_options, &error);
+   collection = mongoc_database_create_collection (
+      database, capped_name, &capped_options, &error);
    ASSERT_OR_PRINT (collection, error);
    mongoc_collection_destroy (collection);
 
-   collection = mongoc_database_create_collection (database, autoindexid_name,
-                                                   &autoindexid_options,
-                                                   &error);
+   collection = mongoc_database_create_collection (
+      database, autoindexid_name, &autoindexid_options, &error);
    ASSERT_OR_PRINT (collection, error);
    mongoc_collection_destroy (collection);
 
-   collection = mongoc_database_create_collection (database, noopts_name,
-                                                   &noopts_options, &error);
+   collection = mongoc_database_create_collection (
+      database, noopts_name, &noopts_options, &error);
    ASSERT_OR_PRINT (collection, error);
    mongoc_collection_destroy (collection);
 
@@ -576,10 +574,12 @@ test_get_collection_info (void)
     * test w/o filters for us. */
 
    /* Filter on an exact match of name */
+   BEGIN_IGNORE_DEPRECATIONS
    cursor = mongoc_database_find_collections (database, &name_filter, &error);
-   assert (cursor);
-   assert (!error.domain);
-   assert (!error.code);
+   END_IGNORE_DEPRECATIONS
+   BSON_ASSERT (cursor);
+   BSON_ASSERT (!error.domain);
+   BSON_ASSERT (!error.code);
 
    while (mongoc_cursor_next (cursor, &doc)) {
       if (bson_iter_init (&col_iter, doc) &&
@@ -587,20 +587,19 @@ test_get_collection_info (void)
           BSON_ITER_HOLDS_UTF8 (&col_iter) &&
           (name = bson_iter_utf8 (&col_iter, NULL))) {
          ++num_infos;
-         assert (0 == strcmp (name, noopts_name));
+         BSON_ASSERT (0 == strcmp (name, noopts_name));
       } else {
-         assert (false);
+         BSON_ASSERT (false);
       }
    }
 
-   assert (1 == num_infos);
+   BSON_ASSERT (1 == num_infos);
 
    mongoc_cursor_destroy (cursor);
 
-   ASSERT_OR_PRINT (mongoc_database_drop (database, &error),
-                    error);
-   assert (!error.domain);
-   assert (!error.code);
+   ASSERT_OR_PRINT (mongoc_database_drop (database, &error), error);
+   BSON_ASSERT (!error.domain);
+   BSON_ASSERT (!error.code);
 
    bson_free (capped_name);
    bson_free (noopts_name);
@@ -610,6 +609,192 @@ test_get_collection_info (void)
    mongoc_client_destroy (client);
 }
 
+static void
+test_get_collection_info_regex (void)
+{
+   mongoc_database_t *database;
+   mongoc_collection_t *collection;
+   mongoc_client_t *client;
+   mongoc_cursor_t *cursor;
+   bson_error_t error = {0};
+   bson_iter_t col_iter;
+   bson_t name_filter = BSON_INITIALIZER;
+   const bson_t *doc;
+   char *dbname;
+
+   client = test_framework_client_new ();
+   BSON_ASSERT (client);
+
+   dbname = gen_collection_name ("test_get_collection_info_regex");
+   database = mongoc_client_get_database (client, dbname);
+   mongoc_database_drop_with_opts (database, NULL, NULL);
+
+   collection =
+      mongoc_database_create_collection (database, "abbbc", NULL, &error);
+   ASSERT_OR_PRINT (collection, error);
+   mongoc_collection_destroy (collection);
+
+   collection =
+      mongoc_database_create_collection (database, "foo", NULL, &error);
+   ASSERT_OR_PRINT (collection, error);
+
+   BSON_APPEND_REGEX (&name_filter, "name", "ab+c", NULL);
+
+   BEGIN_IGNORE_DEPRECATIONS
+   cursor = mongoc_database_find_collections (database, &name_filter, &error);
+   END_IGNORE_DEPRECATIONS
+
+   if (test_framework_max_wire_version_at_least (3)) {
+      BSON_ASSERT (cursor);
+      BSON_ASSERT (!error.domain);
+      BSON_ASSERT (!error.code);
+
+      BSON_ASSERT (mongoc_cursor_next (cursor, &doc));
+      BSON_ASSERT (bson_iter_init_find (&col_iter, doc, "name"));
+      BSON_ASSERT (0 == strcmp (bson_iter_utf8 (&col_iter, NULL), "abbbc"));
+
+      /* only one match */
+      BSON_ASSERT (!mongoc_cursor_next (cursor, &doc));
+      ASSERT_OR_PRINT (!mongoc_cursor_error (cursor, &error), error);
+      mongoc_cursor_destroy (cursor);
+   } else {
+      BSON_ASSERT (!cursor);
+      /* MongoDB 2.6 doesn't allow regex */
+      ASSERT_ERROR_CONTAINS (error,
+                             MONGOC_ERROR_NAMESPACE,
+                             MONGOC_ERROR_NAMESPACE_INVALID_FILTER_TYPE,
+                             "filter on name can only be a string");
+   }
+
+   mongoc_collection_destroy (collection);
+   bson_free (dbname);
+   mongoc_database_destroy (database);
+   mongoc_client_destroy (client);
+}
+
+static void
+test_get_collection_info_with_opts_regex (void)
+{
+   mongoc_database_t *database;
+   mongoc_collection_t *collection;
+   mongoc_client_t *client;
+   mongoc_cursor_t *cursor;
+   bson_error_t error = {0};
+   bson_iter_t col_iter;
+   bson_t opts = BSON_INITIALIZER;
+   bson_t name_filter;
+   const bson_t *doc;
+   char *dbname;
+
+   client = test_framework_client_new ();
+   BSON_ASSERT (client);
+
+   dbname = gen_collection_name ("test_get_collection_info_regex");
+   database = mongoc_client_get_database (client, dbname);
+   mongoc_database_drop_with_opts (database, NULL, NULL);
+
+   collection =
+      mongoc_database_create_collection (database, "abbbc", NULL, &error);
+   ASSERT_OR_PRINT (collection, error);
+   mongoc_collection_destroy (collection);
+
+   collection =
+      mongoc_database_create_collection (database, "foo", NULL, &error);
+   ASSERT_OR_PRINT (collection, error);
+
+   BSON_APPEND_DOCUMENT_BEGIN (&opts, "filter", &name_filter);
+   BSON_APPEND_REGEX (&name_filter, "name", "ab+c", NULL);
+   bson_append_document_end (&opts, &name_filter);
+
+   cursor = mongoc_database_find_collections_with_opts (database, &opts);
+   BSON_ASSERT (cursor);
+
+   if (test_framework_max_wire_version_at_least (3)) {
+      BSON_ASSERT (!error.domain);
+      BSON_ASSERT (!error.code);
+
+      BSON_ASSERT (mongoc_cursor_next (cursor, &doc));
+      BSON_ASSERT (bson_iter_init_find (&col_iter, doc, "name"));
+      BSON_ASSERT (0 == strcmp (bson_iter_utf8 (&col_iter, NULL), "abbbc"));
+
+      /* only one match */
+      BSON_ASSERT (!mongoc_cursor_next (cursor, &doc));
+      ASSERT_OR_PRINT (!mongoc_cursor_error (cursor, &error), error);
+   } else {
+      /* MongoDB 2.6 doesn't allow regex */
+      BSON_ASSERT (mongoc_cursor_error (cursor, &error));
+      ASSERT_ERROR_CONTAINS (error,
+                             MONGOC_ERROR_NAMESPACE,
+                             MONGOC_ERROR_NAMESPACE_INVALID_FILTER_TYPE,
+                             "filter on name can only be a string");
+   }
+
+   mongoc_cursor_destroy (cursor);
+   bson_destroy (&opts);
+   mongoc_collection_destroy (collection);
+   bson_free (dbname);
+   mongoc_database_destroy (database);
+   mongoc_client_destroy (client);
+}
+
+static void
+_test_get_collection_info_getmore ()
+{
+   mock_server_t *server;
+   mongoc_client_t *client;
+   mongoc_database_t *database;
+   future_t *future;
+   request_t *request;
+   char **names;
+
+   server = mock_server_with_autoismaster (WIRE_VERSION_FIND_CMD);
+   mock_server_run (server);
+   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   database = mongoc_client_get_database (client, "db");
+   future =
+      future_database_get_collection_names_with_opts (database, NULL, NULL);
+
+   request = mock_server_receives_command (
+      server, "db", MONGOC_QUERY_SLAVE_OK, "{'listCollections': 1}");
+
+   mock_server_replies_simple (request,
+                               "{'ok': 1,"
+                               " 'cursor': {"
+                               "    'id': {'$numberLong': '123'},"
+                               "    'ns': 'db.$cmd.listCollections',"
+                               "    'firstBatch': [{'name': 'a'}]}}");
+   request_destroy (request);
+   request =
+      mock_server_receives_command (server,
+                                    "db",
+                                    MONGOC_QUERY_SLAVE_OK,
+                                    "{'getMore': {'$numberLong': '123'},"
+                                    " 'collection': '$cmd.listCollections'}");
+
+   mock_server_replies_simple (request,
+                               "{'ok': 1,"
+                               " 'cursor': {"
+                               "    'id': {'$numberLong': '0'},"
+                               "    'ns': 'db.$cmd.listCollections',"
+                               "    'nextBatch': []}}");
+   request_destroy (request);
+   names = future_get_char_ptr_ptr (future);
+   BSON_ASSERT (names);
+   ASSERT_CMPSTR (names[0], "a");
+
+   bson_strfreev (names);
+   future_destroy (future);
+   mongoc_database_destroy (database);
+   mongoc_client_destroy (client);
+   mock_server_destroy (server);
+}
+
+static void
+test_get_collection_info_getmore_cmd (void)
+{
+   _test_get_collection_info_getmore ();
+}
+
 static void
 test_get_collection (void)
 {
@@ -621,7 +806,7 @@ test_get_collection (void)
    mongoc_collection_t *collection;
 
    client = test_framework_client_new ();
-   assert (client);
+   BSON_ASSERT (client);
 
    database = mongoc_client_get_database (client, "test");
 
@@ -656,7 +841,7 @@ test_get_collection_names (void)
    mongoc_database_t *database;
    mongoc_collection_t *collection;
    mongoc_client_t *client;
-   bson_error_t error = { 0 };
+   bson_error_t error = {0};
    bson_t options;
    int namecount = 0;
 
@@ -673,12 +858,12 @@ test_get_collection_names (void)
    const char *system_prefix = "system.";
 
    client = test_framework_client_new ();
-   assert (client);
+   BSON_ASSERT (client);
 
    dbname = gen_collection_name ("dbtest");
    database = mongoc_client_get_database (client, dbname);
 
-   assert (database);
+   BSON_ASSERT (database);
    bson_free (dbname);
 
    bson_init (&options);
@@ -689,56 +874,55 @@ test_get_collection_names (void)
    name4 = gen_collection_name ("name4");
    name5 = gen_collection_name ("name5");
 
-   collection = mongoc_database_create_collection (database, name1, &options,
-                                                   &error);
-   assert (collection);
+   collection =
+      mongoc_database_create_collection (database, name1, &options, &error);
+   BSON_ASSERT (collection);
    mongoc_collection_destroy (collection);
 
-   collection = mongoc_database_create_collection (database, name2, &options,
-                                                   &error);
-   assert (collection);
+   collection =
+      mongoc_database_create_collection (database, name2, &options, &error);
+   BSON_ASSERT (collection);
    mongoc_collection_destroy (collection);
 
-   collection = mongoc_database_create_collection (database, name3, &options,
-                                                   &error);
-   assert (collection);
+   collection =
+      mongoc_database_create_collection (database, name3, &options, &error);
+   BSON_ASSERT (collection);
    mongoc_collection_destroy (collection);
 
-   collection = mongoc_database_create_collection (database, name4, &options,
-                                                   &error);
-   assert (collection);
+   collection =
+      mongoc_database_create_collection (database, name4, &options, &error);
+   BSON_ASSERT (collection);
    mongoc_collection_destroy (collection);
 
-   collection = mongoc_database_create_collection (database, name5, &options,
-                                                   &error);
-   assert (collection);
+   collection =
+      mongoc_database_create_collection (database, name5, &options, &error);
+   BSON_ASSERT (collection);
    mongoc_collection_destroy (collection);
 
-   names = mongoc_database_get_collection_names (database, &error);
-   assert (!error.domain);
-   assert (!error.code);
+   names =
+      mongoc_database_get_collection_names_with_opts (database, NULL, &error);
+   BSON_ASSERT (!error.domain);
+   BSON_ASSERT (!error.code);
 
    for (name = names; *name; ++name) {
       /* inefficient, but OK for a unit test. */
       curname = *name;
 
-      if (0 == strcmp (curname, name1) ||
-          0 == strcmp (curname, name2) ||
-          0 == strcmp (curname, name3) ||
-          0 == strcmp (curname, name4) ||
+      if (0 == strcmp (curname, name1) || 0 == strcmp (curname, name2) ||
+          0 == strcmp (curname, name3) || 0 == strcmp (curname, name4) ||
           0 == strcmp (curname, name5)) {
          ++namecount;
       } else if (0 ==
                  strncmp (curname, system_prefix, strlen (system_prefix))) {
          /* Collections prefixed with 'system.' are system collections */
       } else {
-         assert (false);
+         BSON_ASSERT (false);
       }
 
       bson_free (curname);
    }
 
-   assert (namecount == 5);
+   BSON_ASSERT (namecount == 5);
 
    bson_free (name1);
    bson_free (name2);
@@ -748,10 +932,9 @@ test_get_collection_names (void)
 
    bson_free (names);
 
-   ASSERT_OR_PRINT (mongoc_database_drop (database, &error),
-                    error);
-   assert (!error.domain);
-   assert (!error.code);
+   ASSERT_OR_PRINT (mongoc_database_drop (database, &error), error);
+   BSON_ASSERT (!error.domain);
+   BSON_ASSERT (!error.code);
 
    mongoc_database_destroy (database);
    mongoc_client_destroy (client);
@@ -763,7 +946,7 @@ test_get_collection_names_error (void)
    mongoc_database_t *database;
    mongoc_client_t *client;
    mock_server_t *server;
-   bson_error_t error = { 0 };
+   bson_error_t error = {0};
    bson_t b = BSON_INITIALIZER;
    future_t *future;
    request_t *request;
@@ -772,20 +955,20 @@ test_get_collection_names_error (void)
    capture_logs (true);
 
    server = mock_server_new ();
-   mock_server_auto_ismaster (server, "{'ismaster': true,"
-                                       " 'maxWireVersion': 3}");
+   mock_server_auto_ismaster (server,
+                              "{'ismaster': true,"
+                              " 'maxWireVersion': 3}");
    mock_server_run (server);
    client = mongoc_client_new_from_uri (mock_server_get_uri (server));
 
    database = mongoc_client_get_database (client, "test");
-   future = future_database_get_collection_names (database, &error);
-   request = mock_server_receives_command (server,
-                                            "test",
-                                            MONGOC_QUERY_SLAVE_OK,
-                                            "{'listCollections': 1}");
+   future =
+      future_database_get_collection_names_with_opts (database, NULL, &error);
+   request = mock_server_receives_command (
+      server, "test", MONGOC_QUERY_SLAVE_OK, "{'listCollections': 1}");
    mock_server_hangs_up (request);
    names = future_get_char_ptr_ptr (future);
-   assert (!names);
+   BSON_ASSERT (!names);
    ASSERT_CMPINT (MONGOC_ERROR_STREAM, ==, error.domain);
    ASSERT_CMPINT (MONGOC_ERROR_STREAM_SOCKET, ==, error.code);
 
@@ -804,7 +987,7 @@ test_get_default_database (void)
    mongoc_client_t *client = mongoc_client_new ("mongodb://host/db_name");
    mongoc_database_t *db = mongoc_client_get_default_database (client);
 
-   assert (!strcmp ("db_name", mongoc_database_get_name (db)));
+   BSON_ASSERT (!strcmp ("db_name", mongoc_database_get_name (db)));
 
    mongoc_database_destroy (db);
    mongoc_client_destroy (client);
@@ -813,7 +996,7 @@ test_get_default_database (void)
    client = mongoc_client_new ("mongodb://host/");
    db = mongoc_client_get_default_database (client);
 
-   assert (!db);
+   BSON_ASSERT (!db);
 
    mongoc_client_destroy (client);
 }
@@ -821,29 +1004,44 @@ test_get_default_database (void)
 void
 test_database_install (TestSuite *suite)
 {
-   TestSuite_AddLive (suite, "/Database/create_with_write_concern",
+   TestSuite_AddLive (suite,
+                      "/Database/create_with_write_concern",
                       test_create_with_write_concern);
    TestSuite_AddLive (suite, "/Database/copy", test_copy);
    TestSuite_AddLive (suite, "/Database/has_collection", test_has_collection);
    TestSuite_AddLive (suite, "/Database/command", test_command);
-   TestSuite_Add (suite, "/Database/command/read_prefs/simple/single",
-                  test_db_command_simple_read_prefs_single);
-   TestSuite_Add (suite, "/Database/command/read_prefs/simple/pooled",
-                  test_db_command_simple_read_prefs_pooled);
-   TestSuite_Add (suite, "/Database/command/read_prefs/single",
-                  test_db_command_read_prefs_single);
-   TestSuite_Add (suite, "/Database/command/read_prefs/pooled",
-                  test_db_command_read_prefs_pooled);
+   TestSuite_AddMockServerTest (suite,
+                                "/Database/command/read_prefs/simple/single",
+                                test_db_command_simple_read_prefs_single);
+   TestSuite_AddMockServerTest (suite,
+                                "/Database/command/read_prefs/simple/pooled",
+                                test_db_command_simple_read_prefs_pooled);
+   TestSuite_AddMockServerTest (suite,
+                                "/Database/command/read_prefs/single",
+                                test_db_command_read_prefs_single);
+   TestSuite_AddMockServerTest (suite,
+                                "/Database/command/read_prefs/pooled",
+                                test_db_command_read_prefs_pooled);
    TestSuite_AddLive (suite, "/Database/drop", test_drop);
-   TestSuite_AddLive (suite, "/Database/create_collection", test_create_collection);
-   TestSuite_AddLive (suite, "/Database/get_collection_info",
-                  test_get_collection_info);
-   TestSuite_AddLive (suite, "/Database/get_collection",
-                  test_get_collection);
-   TestSuite_AddLive (suite, "/Database/get_collection_names",
-                  test_get_collection_names);
-   TestSuite_AddLive (suite, "/Database/get_collection_names_error",
-                  test_get_collection_names_error);
-   TestSuite_Add (suite, "/Database/get_default_database",
-                  test_get_default_database);
+   TestSuite_AddLive (
+      suite, "/Database/create_collection", test_create_collection);
+   TestSuite_AddLive (
+      suite, "/Database/get_collection_info", test_get_collection_info);
+   TestSuite_AddLive (suite,
+                      "/Database/get_collection_info_regex",
+                      test_get_collection_info_regex);
+   TestSuite_AddLive (suite,
+                      "/Database/get_collection_info_with_opts_regex",
+                      test_get_collection_info_with_opts_regex);
+   TestSuite_AddMockServerTest (suite,
+                                "/Database/get_collection/getmore_cmd",
+                                test_get_collection_info_getmore_cmd);
+   TestSuite_AddLive (suite, "/Database/get_collection", test_get_collection);
+   TestSuite_AddLive (
+      suite, "/Database/get_collection_names", test_get_collection_names);
+   TestSuite_AddMockServerTest (suite,
+                                "/Database/get_collection_names_error",
+                                test_get_collection_names_error);
+   TestSuite_Add (
+      suite, "/Database/get_default_database", test_get_default_database);
 }
