@@ -1,6 +1,8 @@
 #include <mongoc.h>
 #define MONGOC_INSIDE
 #include <mongoc-gridfs-file-private.h>
+#include <mongoc-client-private.h>
+
 #undef MONGOC_INSIDE
 
 #include "test-libmongoc.h"
@@ -12,24 +14,26 @@
 
 
 static mongoc_gridfs_t *
-get_test_gridfs (mongoc_client_t *client,
-                 const char      *name,
-                 bson_error_t    *error)
+get_test_gridfs (mongoc_client_t *client, const char *name, bson_error_t *error)
 {
    char *gen;
-   char n [48];
+   char n[48];
+   mongoc_database_t *db;
 
    gen = gen_collection_name ("fs");
    bson_snprintf (n, sizeof n, "%s_%s", gen, name);
    bson_free (gen);
 
-   mongoc_database_drop (mongoc_client_get_database (client, "test"), NULL);
+   db = mongoc_client_get_database (client, "test");
+   mongoc_database_drop (db, NULL);
+   mongoc_database_destroy (db);
+
    return mongoc_client_get_gridfs (client, "test", NULL, error);
 }
 
+
 bool
-drop_collections (mongoc_gridfs_t *gridfs,
-                  bson_error_t    *error)
+drop_collections (mongoc_gridfs_t *gridfs, bson_error_t *error)
 {
    return (mongoc_collection_drop (mongoc_gridfs_get_files (gridfs), error) &&
            mongoc_collection_drop (mongoc_gridfs_get_chunks (gridfs), error));
@@ -37,8 +41,7 @@ drop_collections (mongoc_gridfs_t *gridfs,
 
 
 static void
-_check_index (mongoc_collection_t *collection,
-              const char          *index_json)
+_check_index (mongoc_collection_t *collection, const char *index_json)
 {
    mongoc_cursor_t *cursor;
    bson_error_t error;
@@ -73,6 +76,35 @@ _check_index (mongoc_collection_t *collection,
 }
 
 
+static mongoc_gridfs_t *
+_get_gridfs (mock_server_t *server, mongoc_client_t *client)
+{
+   future_t *future;
+   bson_error_t error;
+   request_t *request;
+   mongoc_gridfs_t *gridfs;
+
+   /* gridfs ensures two indexes */
+   future = future_client_get_gridfs (client, "db", NULL, &error);
+   request = mock_server_receives_command (
+      server, "db", MONGOC_QUERY_NONE, "{'createIndexes': 'fs.chunks'}");
+
+   mock_server_replies_ok_and_destroys (request);
+
+   request = mock_server_receives_command (
+      server, "db", MONGOC_QUERY_NONE, "{'createIndexes': 'fs.files'}");
+
+   mock_server_replies_ok_and_destroys (request);
+
+   gridfs = future_get_mongoc_gridfs_ptr (future);
+   ASSERT (gridfs);
+
+   future_destroy (future);
+
+   return gridfs;
+}
+
+
 static void
 test_create (void)
 {
@@ -119,7 +151,7 @@ test_remove (void)
 {
    mongoc_gridfs_t *gridfs;
    mongoc_gridfs_file_t *file;
-   mongoc_gridfs_file_opt_t opts = { 0 };
+   mongoc_gridfs_file_opt_t opts = {0};
    mongoc_client_t *client;
    bson_error_t error;
    char name[32];
@@ -127,8 +159,8 @@ test_remove (void)
    client = test_framework_client_new ();
    ASSERT (client);
 
-   ASSERT_OR_PRINT (gridfs = mongoc_client_get_gridfs (
-      client, "test", "foo", &error), error);
+   ASSERT_OR_PRINT (
+      gridfs = mongoc_client_get_gridfs (client, "test", "foo", &error), error);
 
    mongoc_gridfs_drop (gridfs, &error);
 
@@ -157,7 +189,7 @@ static void
 prep_files (mongoc_gridfs_t *gridfs)
 {
    mongoc_gridfs_file_t *file;
-   mongoc_gridfs_file_opt_t opt = { 0 };
+   mongoc_gridfs_file_opt_t opt = {0};
    char buf[100];
    int i = 0;
 
@@ -206,7 +238,8 @@ test_list (void)
    while ((file = mongoc_gridfs_file_list_next (list))) {
       bson_snprintf (buf, sizeof buf, "file.%d", i++);
 
-      ASSERT_CMPINT (strcmp (mongoc_gridfs_file_get_filename (file), buf), ==, 0);
+      ASSERT_CMPINT (
+         strcmp (mongoc_gridfs_file_get_filename (file), buf), ==, 0);
 
       mongoc_gridfs_file_destroy (file);
    }
@@ -218,14 +251,16 @@ test_list (void)
    ASSERT_OR_PRINT (file = mongoc_gridfs_find_one (gridfs, &query, &error),
                     error);
 
-   ASSERT_CMPINT (strcmp (mongoc_gridfs_file_get_filename (file), "file.1"), ==, 0);
+   ASSERT_CMPINT (
+      strcmp (mongoc_gridfs_file_get_filename (file), "file.1"), ==, 0);
    mongoc_gridfs_file_destroy (file);
 
    ASSERT_OR_PRINT (
       file = mongoc_gridfs_find_one_by_filename (gridfs, "file.1", &error),
       error);
 
-   ASSERT_CMPINT (strcmp (mongoc_gridfs_file_get_filename (file), "file.1"), ==, 0);
+   ASSERT_CMPINT (
+      strcmp (mongoc_gridfs_file_get_filename (file), "file.1"), ==, 0);
    mongoc_gridfs_file_destroy (file);
 
    drop_collections (gridfs, &error);
@@ -266,15 +301,12 @@ test_find_with_opts (void)
    mongoc_gridfs_file_destroy (file);
 
    file = mongoc_gridfs_file_list_next (list);
-   ASSERT (!file);  /* done */
+   ASSERT (!file); /* done */
    ASSERT (!mongoc_gridfs_file_list_error (list, &error));
    mongoc_gridfs_file_list_destroy (list);
 
    file = mongoc_gridfs_find_one_with_opts (
-      gridfs,
-      tmp_bson (NULL),
-      tmp_bson ("{'sort': {'filename': -1}}"),
-      &error);
+      gridfs, tmp_bson (NULL), tmp_bson ("{'sort': {'filename': -1}}"), &error);
 
    ASSERT_OR_PRINT (file, error);
    /* file.2 is first, according to sort order */
@@ -282,20 +314,98 @@ test_find_with_opts (void)
    mongoc_gridfs_file_destroy (file);
 
    file = mongoc_gridfs_find_one_with_opts (
-      gridfs,
-      tmp_bson ("{'x': {'$bad_operator': 1}}"),
-      NULL,
-      &error);
+      gridfs, tmp_bson ("{'x': {'$bad_operator': 1}}"), NULL, &error);
 
    ASSERT (!file);
    ASSERT_CMPINT (error.domain, ==, MONGOC_ERROR_SERVER);
 
+   /* ensure "error" is cleared if we successfully find no file */
+   file = mongoc_gridfs_find_one_with_opts (
+      gridfs, tmp_bson ("{'x': 'doesntexist'}"), NULL, &error);
+
+   ASSERT (!file);
+   ASSERT_CMPINT (error.domain, ==, 0);
+   ASSERT_CMPINT (error.code, ==, 0);
+   ASSERT_CMPSTR (error.message, "");
+
    drop_collections (gridfs, &error);
    mongoc_gridfs_destroy (gridfs);
    mongoc_client_destroy (client);
 }
 
 
+/* mongoc_gridfs_find_one_with_opts uses limit 1, no matter what's in "opts" */
+static void
+test_find_one_with_opts_limit (void)
+{
+   mock_server_t *server;
+   mongoc_client_t *client;
+   mongoc_gridfs_t *gridfs;
+   mongoc_gridfs_file_t *file;
+   bson_error_t error;
+   future_t *future;
+   request_t *request;
+
+   server = mock_server_with_autoismaster (WIRE_VERSION_FIND_CMD);
+   mock_server_run (server);
+   client = mongoc_client_new_from_uri (mock_server_get_uri (server));
+   mongoc_client_set_error_api (client, 2);
+
+   gridfs = _get_gridfs (server, client);
+
+   future =
+      future_gridfs_find_one_with_opts (gridfs, tmp_bson ("{}"), NULL, &error);
+
+   request = mock_server_receives_command (
+      server,
+      "db",
+      MONGOC_QUERY_SLAVE_OK,
+      "{'find': 'fs.files', 'filter': {}, 'limit': 1}");
+
+   mock_server_replies_to_find (request,
+                                MONGOC_QUERY_SLAVE_OK,
+                                0 /* cursor_id */,
+                                1 /* num returned */,
+                                "db.fs.files",
+                                "{'_id': 1, 'length': 1, 'chunkSize': 1}",
+                                true /* is_command */);
+
+   file = future_get_mongoc_gridfs_file_ptr (future);
+   ASSERT (file);
+
+   mongoc_gridfs_file_destroy (file);
+   future_destroy (future);
+   request_destroy (request);
+
+   future = future_gridfs_find_one_with_opts (
+      gridfs, tmp_bson ("{}"), tmp_bson ("{'limit': 2}"), &error);
+
+   request = mock_server_receives_command (
+      server,
+      "db",
+      MONGOC_QUERY_SLAVE_OK,
+      "{'find': 'fs.files', 'filter': {}, 'limit': 1}");
+
+   mock_server_replies_to_find (request,
+                                MONGOC_QUERY_SLAVE_OK,
+                                0 /* cursor_id */,
+                                1 /* num returned */,
+                                "db.fs.files",
+                                "{'_id': 1, 'length': 1, 'chunkSize': 1}",
+                                true /* is_command */);
+
+   file = future_get_mongoc_gridfs_file_ptr (future);
+   ASSERT (file);
+
+   mongoc_gridfs_file_destroy (file);
+   future_destroy (future);
+   request_destroy (request);
+   mongoc_gridfs_destroy (gridfs);
+   mongoc_client_destroy (client);
+   mock_server_destroy (server);
+}
+
+
 static void
 test_properties (void)
 {
@@ -316,17 +426,29 @@ test_properties (void)
    mongoc_gridfs_drop (gridfs, &error);
 
    /* the C Driver sets _id to an ObjectId, but other drivers can do anything */
-   doc_in = BCON_NEW (
-         "_id", BCON_INT32 (1),
-         "md5", BCON_UTF8 ("md5"),
-         "filename", BCON_UTF8 ("filename"),
-         "contentType", BCON_UTF8 ("content_type"),
-         "aliases", "[", BCON_UTF8 ("alias0"), BCON_UTF8 ("alias1"), "]",
-         "metadata", "{", "key", BCON_UTF8 ("value"), "}",
-         "chunkSize", BCON_INT32 (100));
+   doc_in = BCON_NEW ("_id",
+                      BCON_INT32 (1),
+                      "md5",
+                      BCON_UTF8 ("md5"),
+                      "filename",
+                      BCON_UTF8 ("filename"),
+                      "contentType",
+                      BCON_UTF8 ("content_type"),
+                      "aliases",
+                      "[",
+                      BCON_UTF8 ("alias0"),
+                      BCON_UTF8 ("alias1"),
+                      "]",
+                      "metadata",
+                      "{",
+                      "key",
+                      BCON_UTF8 ("value"),
+                      "}",
+                      "chunkSize",
+                      BCON_INT32 (100));
 
-   ASSERT (mongoc_collection_insert (mongoc_gridfs_get_files (gridfs),
-                                     MONGOC_INSERT_NONE, doc_in, NULL, NULL));
+   ASSERT (mongoc_collection_insert_one (
+      mongoc_gridfs_get_files (gridfs), doc_in, NULL, NULL, NULL));
 
    list = mongoc_gridfs_find (gridfs, &query);
    file = mongoc_gridfs_file_list_next (list);
@@ -337,9 +459,11 @@ test_properties (void)
    ASSERT_CMPSTR ("md5", mongoc_gridfs_file_get_md5 (file));
    ASSERT_CMPSTR ("filename", mongoc_gridfs_file_get_filename (file));
    ASSERT_CMPSTR ("content_type", mongoc_gridfs_file_get_content_type (file));
-   ASSERT (BCON_EXTRACT ((bson_t *)mongoc_gridfs_file_get_aliases (file),
-                         "0", BCONE_UTF8 (alias0),
-                         "1", BCONE_UTF8 (alias1)));
+   ASSERT (BCON_EXTRACT ((bson_t *) mongoc_gridfs_file_get_aliases (file),
+                         "0",
+                         BCONE_UTF8 (alias0),
+                         "1",
+                         BCONE_UTF8 (alias1)));
 
    ASSERT_CMPSTR ("alias0", alias0);
    ASSERT_CMPSTR ("alias1", alias1);
@@ -356,8 +480,12 @@ test_properties (void)
 static void
 test_create_from_stream (void)
 {
+   int64_t start;
+   int64_t now;
    mongoc_gridfs_t *gridfs;
    mongoc_gridfs_file_t *file;
+   bson_t *filter;
+   mongoc_gridfs_file_t *file2;
    mongoc_stream_t *stream;
    mongoc_client_t *client;
    bson_error_t error;
@@ -370,7 +498,9 @@ test_create_from_stream (void)
 
    mongoc_gridfs_drop (gridfs, &error);
 
-   stream = mongoc_stream_file_new_for_path (BINARY_DIR"/gridfs.dat", O_RDONLY, 0);
+   start = ((int64_t) time (NULL)) * 1000;
+   stream =
+      mongoc_stream_file_new_for_path (BINARY_DIR "/gridfs.dat", O_RDONLY, 0);
    ASSERT_OR_PRINT_ERRNO (stream, errno);
    ASSERT (stream);
 
@@ -378,11 +508,20 @@ test_create_from_stream (void)
    ASSERT (file);
    ASSERT (mongoc_gridfs_file_save (file));
 
+   now = ((int64_t) time (NULL)) * 1000;
+
+   filter = tmp_bson (NULL);
+   BSON_APPEND_VALUE (filter, "_id", mongoc_gridfs_file_get_id (file));
+   file2 = mongoc_gridfs_find_one_with_opts (gridfs, filter, NULL, &error);
+   ASSERT_OR_PRINT (file2, error);
+   ASSERT_CMPINT64 (start, <=, mongoc_gridfs_file_get_upload_date (file2));
+   ASSERT_CMPINT64 (now, >=, mongoc_gridfs_file_get_upload_date (file2));
+
+   mongoc_gridfs_file_destroy (file2);
    mongoc_gridfs_file_destroy (file);
 
    drop_collections (gridfs, &error);
    mongoc_gridfs_destroy (gridfs);
-
    mongoc_client_destroy (client);
 }
 
@@ -402,20 +541,24 @@ test_seek (void)
 
    mongoc_gridfs_drop (gridfs, &error);
 
-   stream = mongoc_stream_file_new_for_path (BINARY_DIR"/gridfs-large.dat", O_RDONLY, 0);
+   stream = mongoc_stream_file_new_for_path (
+      BINARY_DIR "/gridfs-large.dat", O_RDONLY, 0);
 
    file = mongoc_gridfs_create_file_from_stream (gridfs, stream, NULL);
    ASSERT (file);
    ASSERT (mongoc_gridfs_file_save (file));
 
    ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_SET), ==, 0);
-   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)0);
+   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t) 0);
 
-   ASSERT_CMPINT (mongoc_gridfs_file_seek (file, file->chunk_size + 1, SEEK_CUR), ==, 0);
-   ASSERT_CMPINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)(file->chunk_size + 1));
+   ASSERT_CMPINT (
+      mongoc_gridfs_file_seek (file, file->chunk_size + 1, SEEK_CUR), ==, 0);
+   ASSERT_CMPINT64 (
+      mongoc_gridfs_file_tell (file), ==, (uint64_t) (file->chunk_size + 1));
 
    ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_END), ==, 0);
-   ASSERT_CMPINT64 (mongoc_gridfs_file_tell (file), ==, mongoc_gridfs_file_get_length (file));
+   ASSERT_CMPINT64 (
+      mongoc_gridfs_file_tell (file), ==, mongoc_gridfs_file_get_length (file));
 
    mongoc_gridfs_file_destroy (file);
 
@@ -453,7 +596,8 @@ test_read (void)
 
    mongoc_gridfs_drop (gridfs, &error);
 
-   stream = mongoc_stream_file_new_for_path (BINARY_DIR"/gridfs-large.dat", O_RDONLY, 0);
+   stream = mongoc_stream_file_new_for_path (
+      BINARY_DIR "/gridfs-large.dat", O_RDONLY, 0);
 
    file = mongoc_gridfs_create_file_from_stream (gridfs, stream, NULL);
    ASSERT (file);
@@ -471,21 +615,23 @@ test_read (void)
    ASSERT_MEMCMP (iov[0].iov_base, "acon ipsum", 10);
    ASSERT_MEMCMP (iov[1].iov_base, " dolor sit", 10);
 
-   ASSERT_CMPINT (mongoc_gridfs_file_seek (file, file->chunk_size-1, SEEK_SET), ==, 0);
+   ASSERT_CMPINT (
+      mongoc_gridfs_file_seek (file, file->chunk_size - 1, SEEK_SET), ==, 0);
    r = mongoc_gridfs_file_readv (file, iov, 2, 20, 0);
 
    ASSERT_CMPSSIZE_T (r, ==, twenty);
-   ASSERT_CMPINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)(file->chunk_size+19));
+   ASSERT_CMPINT64 (
+      mongoc_gridfs_file_tell (file), ==, (uint64_t) (file->chunk_size + 19));
    ASSERT_MEMCMP (iov[0].iov_base, "turducken ", 10);
    ASSERT_MEMCMP (iov[1].iov_base, "spare ribs", 10);
 
-   assert (mongoc_gridfs_file_seek (file, 20, SEEK_END) == 0);
+   BSON_ASSERT (mongoc_gridfs_file_seek (file, 20, SEEK_END) == 0);
    previous_errno = errno;
    r = mongoc_gridfs_file_readv (file, iov, 2, 20, 0);
 
-   assert (errno == previous_errno);
-   assert (r == 0);
-   assert (mongoc_gridfs_file_tell (file) == file->length + 20);
+   BSON_ASSERT (errno == previous_errno);
+   BSON_ASSERT (r == 0);
+   BSON_ASSERT (mongoc_gridfs_file_tell (file) == file->length + 20);
 
    mongoc_gridfs_file_destroy (file);
 
@@ -497,9 +643,7 @@ test_read (void)
 
 
 static void
-_check_chunk_count (mongoc_gridfs_t *gridfs,
-                    int64_t          len,
-                    int64_t          chunk_size)
+_check_chunk_count (mongoc_gridfs_t *gridfs, int64_t len, int64_t chunk_size)
 {
    int64_t expected_chunks;
    int64_t cnt;
@@ -508,8 +652,12 @@ _check_chunk_count (mongoc_gridfs_t *gridfs,
    /* division, rounding up */
    expected_chunks = (len + chunk_size - 1) / chunk_size;
    cnt = mongoc_collection_count (mongoc_gridfs_get_chunks (gridfs),
-                                  MONGOC_QUERY_NONE, tmp_bson (NULL), 0, 0,
-                                  NULL, &error);
+                                  MONGOC_QUERY_NONE,
+                                  tmp_bson (NULL),
+                                  0,
+                                  0,
+                                  NULL,
+                                  &error);
 
    ASSERT_CMPINT64 (expected_chunks, ==, cnt);
 }
@@ -526,16 +674,16 @@ _test_write (bool at_boundary)
    char buf[] = "foo bar";
    char buf2[] = " baz";
    char buf3[1000];
-   char expected[1000] = { 0 };
-   mongoc_gridfs_file_opt_t opt = { 0 };
+   char expected[1000] = {0};
+   mongoc_gridfs_file_opt_t opt = {0};
    mongoc_iovec_t iov[2];
    mongoc_iovec_t riov;
    ssize_t len = sizeof buf + sizeof buf2 - 2;
 
-   iov [0].iov_base = buf;
-   iov [0].iov_len = sizeof (buf) - 1;
-   iov [1].iov_base = buf2;
-   iov [1].iov_len = sizeof (buf2) - 1;
+   iov[0].iov_base = buf;
+   iov[0].iov_len = sizeof (buf) - 1;
+   iov[1].iov_base = buf2;
+   iov[1].iov_len = sizeof (buf2) - 1;
 
    riov.iov_base = buf3;
    riov.iov_len = sizeof (buf3);
@@ -557,20 +705,22 @@ _test_write (bool at_boundary)
    ASSERT_CMPSSIZE_T (r, ==, len);
 
    ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_SET), ==, 0);
-   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)0);
+   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t) 0);
 
    r = mongoc_gridfs_file_readv (file, &riov, 1, len, 0);
    ASSERT_CMPSSIZE_T (r, ==, len);
    ASSERT_CMPINT (memcmp (buf3, "foo bar baz", len), ==, 0);
 
-   ASSERT_CMPINT (mongoc_gridfs_file_seek (file, file->chunk_size, SEEK_SET), ==, 0);
-   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)(file->chunk_size));
+   ASSERT_CMPINT (
+      mongoc_gridfs_file_seek (file, file->chunk_size, SEEK_SET), ==, 0);
+   ASSERT_CMPUINT64 (
+      mongoc_gridfs_file_tell (file), ==, (uint64_t) (file->chunk_size));
 
-   r = mongoc_gridfs_file_writev (file, iov+1, 1, 0);
+   r = mongoc_gridfs_file_writev (file, iov + 1, 1, 0);
    ASSERT_CMPSSIZE_T (r, ==, iov[1].iov_len);
 
    ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_SET), ==, 0);
-   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)0);
+   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t) 0);
 
    r = mongoc_gridfs_file_readv (file, &riov, 1, len, 0);
    ASSERT_CMPSSIZE_T (r, ==, len);
@@ -578,29 +728,30 @@ _test_write (bool at_boundary)
    _check_chunk_count (gridfs, len, file->chunk_size);
 
    /* Test writing beyond the end of the file */
-   assert (mongoc_gridfs_file_seek (file, seek_len, SEEK_END) == 0);
-   assert (mongoc_gridfs_file_tell (file) == file->length + seek_len);
+   BSON_ASSERT (mongoc_gridfs_file_seek (file, seek_len, SEEK_END) == 0);
+   BSON_ASSERT (mongoc_gridfs_file_tell (file) == file->length + seek_len);
 
    r = mongoc_gridfs_file_writev (file, iov, 2, 0);
-   assert (r == len);
-   assert (mongoc_gridfs_file_tell (file) == 2 * len + seek_len);
-   assert (file->length == 2 * len + seek_len);
-   assert (mongoc_gridfs_file_save (file));
+   BSON_ASSERT (r == len);
+   BSON_ASSERT (mongoc_gridfs_file_tell (file) == 2 * len + seek_len);
+   BSON_ASSERT (file->length == 2 * len + seek_len);
+   BSON_ASSERT (mongoc_gridfs_file_save (file));
    _check_chunk_count (gridfs, 2 * len + seek_len, file->chunk_size);
 
-   assert (mongoc_gridfs_file_seek (file, 0, SEEK_SET) == 0);
-   assert (mongoc_gridfs_file_tell (file) == 0);
+   BSON_ASSERT (mongoc_gridfs_file_seek (file, 0, SEEK_SET) == 0);
+   BSON_ASSERT (mongoc_gridfs_file_tell (file) == 0);
 
    r = mongoc_gridfs_file_readv (file, &riov, 1, 2 * len + seek_len, 0);
-   assert (r == 2 * len + seek_len);
+   BSON_ASSERT (r == 2 * len + seek_len);
 
    /* expect file to be like "fo bazr baz\0\0\0\0\0\0foo bar baz" */
    bson_snprintf (expected, strlen ("fo bazr baz") + 1, "fo bazr baz");
    bson_snprintf (expected + strlen ("fo bazr baz") + seek_len,
-                  strlen ("foo bar baz") + 1,"foo bar baz");
+                  strlen ("foo bar baz") + 1,
+                  "foo bar baz");
 
-   assert (memcmp (buf3, expected, (size_t) (2 * len + seek_len)) == 0);
-   assert (mongoc_gridfs_file_save (file));
+   BSON_ASSERT (memcmp (buf3, expected, (size_t) (2 * len + seek_len)) == 0);
+   BSON_ASSERT (mongoc_gridfs_file_save (file));
 
    mongoc_gridfs_file_destroy (file);
 
@@ -636,7 +787,7 @@ test_write_past_end (void)
    ssize_t r;
    char buf[] = "foo";
    char read_buf[2000];
-   mongoc_gridfs_file_opt_t opt = { 0 };
+   mongoc_gridfs_file_opt_t opt = {0};
    mongoc_iovec_t iov[1];
    mongoc_iovec_t riov;
    const size_t len = sizeof (buf) - 1;
@@ -646,8 +797,8 @@ test_write_past_end (void)
    const int64_t expected_chunks = ((delta + len) + (chunk_sz - 1)) / chunk_sz;
    int64_t cnt;
 
-   iov [0].iov_base = buf;
-   iov [0].iov_len = sizeof (buf) - 1;
+   iov[0].iov_base = buf;
+   iov[0].iov_len = sizeof (buf) - 1;
 
    riov.iov_base = read_buf;
    riov.iov_len = sizeof (read_buf);
@@ -675,8 +826,12 @@ test_write_past_end (void)
    mongoc_gridfs_file_save (file);
 
    cnt = mongoc_collection_count (mongoc_gridfs_get_chunks (gridfs),
-                                  MONGOC_QUERY_NONE, tmp_bson (NULL), 0, 0,
-                                  NULL, &error);
+                                  MONGOC_QUERY_NONE,
+                                  tmp_bson (NULL),
+                                  0,
+                                  0,
+                                  NULL,
+                                  &error);
 
    ASSERT_OR_PRINT (cnt != -1, error);
    ASSERT_CMPINT64 (expected_chunks, ==, cnt);
@@ -715,28 +870,29 @@ test_empty (void)
 
    ASSERT_OR_PRINT (gridfs = get_test_gridfs (client, "empty", &error), error);
 
-   stream = mongoc_stream_file_new_for_path (BINARY_DIR"/empty.dat", O_RDONLY, 0);
+   stream =
+      mongoc_stream_file_new_for_path (BINARY_DIR "/empty.dat", O_RDONLY, 0);
    ASSERT_OR_PRINT_ERRNO (stream, errno);
 
    file = mongoc_gridfs_create_file_from_stream (gridfs, stream, NULL);
    ASSERT (file);
 
    ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_SET), ==, 0);
-   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)0);
+   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t) 0);
 
    ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_CUR), ==, 0);
-   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)0);
+   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t) 0);
 
    ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_END), ==, 0);
-   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)0);
+   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t) 0);
 
-   r = mongoc_gridfs_file_writev(file, iov, 1, 0);
+   r = mongoc_gridfs_file_writev (file, iov, 1, 0);
 
    ASSERT_CMPSSIZE_T (r, ==, two);
    ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_SET), ==, 0);
-   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t)0);
+   ASSERT_CMPUINT64 (mongoc_gridfs_file_tell (file), ==, (uint64_t) 0);
 
-   r = mongoc_gridfs_file_readv(file, iov, 1, 2, 0);
+   r = mongoc_gridfs_file_readv (file, iov, 1, 2, 0);
 
    ASSERT_CMPSSIZE_T (r, ==, two);
    ASSERT_CMPINT (strncmp (buf, "hi", 2), ==, 0);
@@ -773,7 +929,8 @@ test_stream (void)
 
    mongoc_gridfs_drop (gridfs, &error);
 
-   in_stream = mongoc_stream_file_new_for_path (BINARY_DIR"/gridfs.dat", O_RDONLY, 0);
+   in_stream =
+      mongoc_stream_file_new_for_path (BINARY_DIR "/gridfs.dat", O_RDONLY, 0);
    ASSERT_OR_PRINT_ERRNO (in_stream, errno);
 
    file = mongoc_gridfs_create_file_from_stream (gridfs, in_stream, NULL);
@@ -783,7 +940,7 @@ test_stream (void)
    stream = mongoc_stream_gridfs_new (file);
 
    r = mongoc_stream_readv (stream, &iov, 1, file->length, 0);
-   ASSERT_CMPINT64 ((int64_t)r, ==, file->length);
+   ASSERT_CMPINT64 ((int64_t) r, ==, file->length);
 
    /* cleanup */
    mongoc_stream_destroy (stream);
@@ -810,7 +967,7 @@ test_long_seek (void *ctx)
    mongoc_gridfs_t *gridfs;
    mongoc_gridfs_file_t *file;
    ssize_t r;
-   mongoc_gridfs_file_opt_t opt = { 0, "filename" };
+   mongoc_gridfs_file_opt_t opt = {0, "filename"};
    mongoc_iovec_t iov;
    char buf[16 * 1024]; /* nothing special about 16k, just a buffer */
    const ssize_t buflen = sizeof (buf);
@@ -838,9 +995,8 @@ test_long_seek (void *ctx)
    /* new file handle */
    mongoc_gridfs_file_save (file);
    mongoc_gridfs_file_destroy (file);
-   file = mongoc_gridfs_find_one (gridfs,
-                                  tmp_bson ("{'filename': 'filename'}"),
-                                  &error);
+   file = mongoc_gridfs_find_one (
+      gridfs, tmp_bson ("{'filename': 'filename'}"), &error);
 
    ASSERT_OR_PRINT (file, error);
 
@@ -885,15 +1041,16 @@ test_remove_by_filename (void)
 {
    mongoc_gridfs_t *gridfs;
    mongoc_gridfs_file_t *file;
-   mongoc_gridfs_file_opt_t opt = { 0 };
+   mongoc_gridfs_file_opt_t opt = {0};
    mongoc_client_t *client;
    bson_error_t error;
 
    client = test_framework_client_new ();
    ASSERT (client);
 
-   ASSERT_OR_PRINT (gridfs = get_test_gridfs (
-      client, "fs_remove_by_filename", &error), error);
+   ASSERT_OR_PRINT (
+      gridfs = get_test_gridfs (client, "fs_remove_by_filename", &error),
+      error);
 
    mongoc_gridfs_drop (gridfs, &error);
 
@@ -935,7 +1092,7 @@ test_missing_chunk (void *ctx)
    mongoc_gridfs_file_t *file;
    mongoc_collection_t *chunks;
    ssize_t r;
-   mongoc_gridfs_file_opt_t opt = { 0, "filename" };
+   mongoc_gridfs_file_opt_t opt = {0, "filename"};
    mongoc_iovec_t iov;
    char buf[16 * 1024]; /* nothing special about 16k, just a buffer */
    const ssize_t buflen = sizeof (buf);
@@ -968,8 +1125,8 @@ test_missing_chunk (void *ctx)
 
    /* chunks have n=0, 1, 2; remove the middle one */
    chunks = mongoc_gridfs_get_chunks (gridfs);
-   ret = mongoc_collection_remove (chunks, MONGOC_REMOVE_NONE,
-                                   tmp_bson ("{'n': 1}"), NULL, &error);
+   ret = mongoc_collection_delete_many (
+      chunks, tmp_bson ("{'n': 1}"), NULL, NULL, &error);
 
    ASSERT_OR_PRINT (ret, error);
 
@@ -995,44 +1152,63 @@ test_missing_chunk (void *ctx)
    mongoc_client_destroy (client);
 }
 
-static mongoc_gridfs_t *
-_get_gridfs (mock_server_t *server,
-             mongoc_client_t *client)
+static void
+test_missing_file (void)
 {
-   future_t *future;
-   bson_error_t error;
-   request_t *request;
+   mongoc_client_t *client;
    mongoc_gridfs_t *gridfs;
+   mongoc_gridfs_file_t *file;
+   bson_error_t error;
+   char buf[] = "contents contents";
+   mongoc_iovec_t iov;
 
-   /* gridfs ensures two indexes */
-   future = future_client_get_gridfs (client, "db", NULL, &error);
-   request = mock_server_receives_command (
-      server,
-      "db",
-      MONGOC_QUERY_NONE,
-      "{'createIndexes': 'fs.chunks'}");
+   iov.iov_base = buf;
+   iov.iov_len = sizeof buf;
 
-   mock_server_replies_ok_and_destroys (request);
+   client = test_framework_client_new ();
+   gridfs =
+      mongoc_client_get_gridfs (client, "test_missing_file", NULL, &error);
+   ASSERT_OR_PRINT (gridfs, error);
 
-   request = mock_server_receives_command (
-      server,
-      "db",
-      MONGOC_QUERY_NONE,
-      "{'createIndexes': 'fs.files'}");
+   file = mongoc_gridfs_create_file (gridfs, NULL);
+   ASSERT_CMPSSIZE_T (
+      mongoc_gridfs_file_writev (file, &iov, 1, 0), ==, (ssize_t) sizeof buf);
+   ASSERT_CMPINT (mongoc_gridfs_file_seek (file, 0, SEEK_SET), ==, 0);
+   BSON_ASSERT (mongoc_gridfs_file_save (file));
 
-   mock_server_replies_ok_and_destroys (request);
+   /* remove the file */
+   BSON_ASSERT (mongoc_gridfs_file_remove (file, &error));
 
-   gridfs = future_get_mongoc_gridfs_ptr (future);
-   ASSERT (gridfs);
+   /* readv fails */
+   ASSERT_CMPSSIZE_T (mongoc_gridfs_file_readv (file, &iov, 1, sizeof buf, 0),
+                      ==,
+                      (ssize_t) -1);
+   BSON_ASSERT (mongoc_gridfs_file_error (file, &error));
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_GRIDFS,
+                          MONGOC_ERROR_GRIDFS_CHUNK_MISSING,
+                          "missing chunk number 0");
 
-   future_destroy (future);
+   memset (&error, 0, sizeof error);
 
-   return gridfs;
+   /* writev fails */
+   ASSERT_CMPSSIZE_T (
+      mongoc_gridfs_file_writev (file, &iov, 1, 0), ==, (ssize_t) -1);
+   BSON_ASSERT (mongoc_gridfs_file_error (file, &error));
+   ASSERT_ERROR_CONTAINS (error,
+                          MONGOC_ERROR_GRIDFS,
+                          MONGOC_ERROR_GRIDFS_CHUNK_MISSING,
+                          "missing chunk number 0");
+
+   mongoc_gridfs_file_destroy (file);
+   mongoc_gridfs_destroy (gridfs);
+   mongoc_client_destroy (client);
 }
 
+
 /* check that user can specify _id of any type for file */
 static void
-test_set_id (void) 
+test_set_id (void)
 {
    mongoc_gridfs_t *gridfs;
    mongoc_client_t *client;
@@ -1041,37 +1217,36 @@ test_set_id (void)
    bson_t *query;
    mongoc_gridfs_file_t *file;
    mongoc_gridfs_file_t *result;
-   mongoc_gridfs_file_opt_t opt = { 0 };
+   mongoc_gridfs_file_opt_t opt = {0};
 
-   /* create new client and grab gridfs handle */ 
+   /* create new client and grab gridfs handle */
    client = test_framework_client_new ();
-   ASSERT (client); 
-   gridfs = mongoc_client_get_gridfs (client, "test", "fs", &error); 
+   ASSERT (client);
+   gridfs = mongoc_client_get_gridfs (client, "test", "fs", &error);
    ASSERT_OR_PRINT (gridfs, error);
-      
+
    /* create bson */
    id.value_type = BSON_TYPE_INT32;
    id.value.v_int32 = 1;
-   
-   /* query for finding file */ 
+
+   /* query for finding file */
    query = tmp_bson ("{'_id': 1}");
- 
+
    /* create new file */
    opt.filename = "test";
    file = mongoc_gridfs_create_file (gridfs, &opt);
-   ASSERT (file); 
-  
+   ASSERT (file);
+
    /* if we find a file with new id, then file_set_id worked */
    ASSERT_OR_PRINT (mongoc_gridfs_file_set_id (file, &id, &error), error);
    ASSERT (mongoc_gridfs_file_save (file));
-   result = mongoc_gridfs_find_one (gridfs, query, 
-                                    &error); 
-   ASSERT_OR_PRINT (result, error); 
+   result = mongoc_gridfs_find_one (gridfs, query, &error);
+   ASSERT_OR_PRINT (result, error);
 
    mongoc_gridfs_file_destroy (result);
-   mongoc_gridfs_file_destroy (file); 
-   mongoc_gridfs_destroy (gridfs); 
-   mongoc_client_destroy (client);  
+   mongoc_gridfs_file_destroy (file);
+   mongoc_gridfs_destroy (gridfs);
+   mongoc_client_destroy (client);
 }
 
 /* check gridfs inherits read / write concern, read prefs from the client */
@@ -1113,7 +1288,9 @@ test_inherit_client_config (void)
    /* test read prefs and read concern */
    future = future_gridfs_find_one (gridfs, tmp_bson ("{}"), &error);
    request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_SLAVE_OK,
+      server,
+      "db",
+      MONGOC_QUERY_SLAVE_OK,
       "{'$query': {'find': 'fs.files', 'readConcern': {'level': 'majority'}},"
       " '$readPreference': {'mode': 'secondary'}}");
 
@@ -1131,13 +1308,17 @@ test_inherit_client_config (void)
    future = future_gridfs_file_remove (file, &error);
 
    request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_NONE,
+      server,
+      "db",
+      MONGOC_QUERY_NONE,
       "{'delete': 'fs.files', 'writeConcern': {'w': 2}}");
 
    mock_server_replies_ok_and_destroys (request);
 
    request = mock_server_receives_command (
-      server, "db", MONGOC_QUERY_NONE,
+      server,
+      "db",
+      MONGOC_QUERY_NONE,
       "{'delete': 'fs.chunks', 'writeConcern': {'w': 2}}");
 
    mock_server_replies_ok_and_destroys (request);
@@ -1153,13 +1334,38 @@ test_inherit_client_config (void)
    mock_server_destroy (server);
 }
 
+static void
+test_find_one_empty (void)
+{
+   mongoc_gridfs_t *gridfs;
+   mongoc_client_t *client;
+   bson_error_t error = {1, 2, "hello"};
+
+   client = test_framework_client_new ();
+   ASSERT_OR_PRINT (gridfs = get_test_gridfs (client, "list", &error), error);
+   ASSERT (!mongoc_gridfs_find_one (
+      gridfs, tmp_bson ("{'x': 'doesntexist'}"), &error));
+
+   /* ensure "error" is cleared if we successfully find no file */
+   ASSERT_CMPINT (error.domain, ==, 0);
+   ASSERT_CMPINT (error.code, ==, 0);
+   ASSERT_CMPSTR (error.message, "");
+
+   mongoc_gridfs_destroy (gridfs);
+   mongoc_client_destroy (client);
+}
+
 void
 test_gridfs_install (TestSuite *suite)
 {
    TestSuite_AddLive (suite, "/GridFS/create", test_create);
-   TestSuite_AddLive (suite, "/GridFS/create_from_stream", test_create_from_stream);
+   TestSuite_AddLive (
+      suite, "/GridFS/create_from_stream", test_create_from_stream);
    TestSuite_AddLive (suite, "/GridFS/list", test_list);
+   TestSuite_AddLive (suite, "/GridFS/find_one_empty", test_find_one_empty);
    TestSuite_AddLive (suite, "/GridFS/find_with_opts", test_find_with_opts);
+   TestSuite_AddMockServerTest (
+      suite, "/GridFS/find_one_with_opts/limit", test_find_one_with_opts_limit);
    TestSuite_AddLive (suite, "/GridFS/properties", test_properties);
    TestSuite_AddLive (suite, "/GridFS/empty", test_empty);
    TestSuite_AddLive (suite, "/GridFS/read", test_read);
@@ -1167,12 +1373,25 @@ test_gridfs_install (TestSuite *suite)
    TestSuite_AddLive (suite, "/GridFS/stream", test_stream);
    TestSuite_AddLive (suite, "/GridFS/remove", test_remove);
    TestSuite_AddLive (suite, "/GridFS/write", test_write);
-   TestSuite_AddLive (suite, "/GridFS/write_at_boundary",
-                      test_write_at_boundary);
+   TestSuite_AddLive (
+      suite, "/GridFS/write_at_boundary", test_write_at_boundary);
    TestSuite_AddLive (suite, "/GridFS/write_past_end", test_write_past_end);
-   TestSuite_AddFull (suite, "/GridFS/test_long_seek", test_long_seek, NULL, NULL, test_framework_skip_if_slow_or_live);
-   TestSuite_AddLive (suite, "/GridFS/remove_by_filename", test_remove_by_filename);
-   TestSuite_AddFull (suite, "/GridFS/missing_chunk", test_missing_chunk, NULL, NULL, test_framework_skip_if_slow_or_live);
-   TestSuite_AddLive (suite, "/GridFS/file_set_id", test_set_id); 
-   TestSuite_Add (suite, "/GridFS/inherit_client_config", test_inherit_client_config);
+   TestSuite_AddFull (suite,
+                      "/GridFS/test_long_seek",
+                      test_long_seek,
+                      NULL,
+                      NULL,
+                      test_framework_skip_if_slow_or_live);
+   TestSuite_AddLive (
+      suite, "/GridFS/remove_by_filename", test_remove_by_filename);
+   TestSuite_AddFull (suite,
+                      "/GridFS/missing_chunk",
+                      test_missing_chunk,
+                      NULL,
+                      NULL,
+                      test_framework_skip_if_slow_or_live);
+   TestSuite_AddLive (suite, "/GridFS/missing_file", test_missing_file);
+   TestSuite_AddLive (suite, "/GridFS/file_set_id", test_set_id);
+   TestSuite_AddMockServerTest (
+      suite, "/GridFS/inherit_client_config", test_inherit_client_config);
 }
