@@ -23,7 +23,10 @@
  * These are the C examples for that page.
  */
 
+/* clang-format off */
 #include <mongoc.h>
+#include <mongoc-util-private.h>
+#include <mongoc-database-private.h>
 
 #include "TestSuite.h"
 #include "test-libmongoc.h"
@@ -77,8 +80,7 @@ test_example_1 (mongoc_database_t *db)
       "uom", BCON_UTF8 ("cm"),
       "}");
 
-   /* MONGOC_INSERT_NONE means "no special options" */
-   r = mongoc_collection_insert (collection, MONGOC_INSERT_NONE, doc, NULL, &error);
+   r = mongoc_collection_insert_one (collection, doc, NULL, NULL, &error);
    bson_destroy (doc);
 
    if (!r) {
@@ -127,7 +129,7 @@ test_example_3 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "item", BCON_UTF8 ("journal"),
       "qty", BCON_INT64 (25),
@@ -213,7 +215,7 @@ test_example_6 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "item", BCON_UTF8 ("journal"),
       "qty", BCON_INT64 (25),
@@ -481,7 +483,7 @@ test_example_14 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "item", BCON_UTF8 ("journal"),
       "qty", BCON_INT64 (25),
@@ -718,7 +720,7 @@ test_example_20 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "item", BCON_UTF8 ("journal"),
       "qty", BCON_INT64 (25),
@@ -1027,7 +1029,7 @@ test_example_29 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "item", BCON_UTF8 ("journal"),
       "instock", "[",
@@ -1350,7 +1352,7 @@ test_example_38 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "_id", BCON_INT64 (1),
       "item", BCON_NULL);
@@ -1468,7 +1470,7 @@ test_example_42 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "item", BCON_UTF8 ("journal"),
       "status", BCON_UTF8 ("A"),
@@ -1909,7 +1911,7 @@ test_example_51 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "item", BCON_UTF8 ("canvas"),
       "qty", BCON_INT64 (100),
@@ -2283,7 +2285,7 @@ test_example_55 (mongoc_database_t *db)
    bson_t reply;
 
    collection = mongoc_database_get_collection (db, "inventory");
-   bulk = mongoc_collection_create_bulk_operation (collection, true, NULL);
+   bulk = mongoc_collection_create_bulk_operation_with_opts (collection, NULL);
    doc = BCON_NEW (
       "item", BCON_UTF8 ("journal"),
       "qty", BCON_INT64 (25),
@@ -2397,8 +2399,7 @@ test_example_57 (mongoc_database_t *db)
    collection = mongoc_database_get_collection (db, "inventory");
    selector = BCON_NEW ("status", BCON_UTF8 ("A"));
 
-   /* MONGOC_REMOVE_NONE means "no special options" */
-   r = mongoc_collection_remove (collection, MONGOC_REMOVE_NONE, selector, NULL, &error);
+   r = mongoc_collection_delete_many (collection, selector, NULL, NULL, &error);
    bson_destroy (selector);
 
    if (!r) {
@@ -2426,7 +2427,7 @@ test_example_58 (mongoc_database_t *db)
    collection = mongoc_database_get_collection (db, "inventory");
    selector = BCON_NEW ("status", BCON_UTF8 ("D"));
 
-   r = mongoc_collection_remove (collection, MONGOC_REMOVE_SINGLE_REMOVE, selector, NULL, &error);
+   r = mongoc_collection_delete_one (collection, selector, NULL, NULL, &error);
    bson_destroy (selector);
 
    if (!r) {
@@ -2454,8 +2455,7 @@ test_example_56 (mongoc_database_t *db)
    collection = mongoc_database_get_collection (db, "inventory");
    selector = BCON_NEW (NULL);
 
-   /* MONGOC_REMOVE_NONE means "no special options" */
-   r = mongoc_collection_remove (collection, MONGOC_REMOVE_NONE, selector, NULL, &error);
+   r = mongoc_collection_delete_many (collection, selector, NULL, NULL, &error);
    bson_destroy (selector);
 
    if (!r) {
@@ -2471,8 +2471,140 @@ done:
 }
 
 
+typedef struct {
+   mongoc_mutex_t lock;
+   mongoc_collection_t *collection;
+   bool done;
+} change_stream_ctx_t;
+
+
+static void *
+insert_docs (void *p)
+{
+   change_stream_ctx_t *ctx = (change_stream_ctx_t *) p;
+   bson_t doc = BSON_INITIALIZER;
+   bson_error_t error;
+   bool r;
+
+   while (true) {
+      mongoc_mutex_lock (&ctx->lock);
+      r = mongoc_collection_insert (
+         ctx->collection, MONGOC_INSERT_NONE, &doc, NULL, &error);
+      ASSERT_OR_PRINT (r, error);
+      if (ctx->done) {
+         mongoc_mutex_unlock (&ctx->lock);
+         return 0;
+      }
+
+      mongoc_mutex_unlock (&ctx->lock);
+      _mongoc_usleep (100 * 1000);  /* 100 ms */
+   }
+}
+
+
 static void
-test_sample_commands (void *ctx)
+test_sample_change_stream_command (sample_command_fn_t fn,
+                                   mongoc_database_t *db)
+{
+   mongoc_client_t *client;
+   change_stream_ctx_t ctx;
+   mongoc_thread_t thread;
+   int r;
+
+   /* change streams require a replica set running MongoDB 3.6+ */
+   if (test_framework_skip_if_not_rs_version_6 () &&
+       test_framework_skip_if_slow ()) {
+
+      /* separate client for the background thread */
+      client = test_framework_client_new ();
+
+      mongoc_mutex_init (&ctx.lock);
+      ctx.collection = mongoc_client_get_collection (
+         client, db->name, "inventory");
+      ctx.done = false;
+
+      r = mongoc_thread_create (&thread, insert_docs, (void *) &ctx);
+      ASSERT_OR_PRINT_ERRNO (r == 0, r);
+
+      capture_logs (true);
+      fn (db);
+      ASSERT_NO_CAPTURED_LOGS ("change stream examples");
+
+      mongoc_mutex_lock (&ctx.lock);
+      ctx.done = true;
+      mongoc_mutex_unlock (&ctx.lock);
+      mongoc_thread_join (thread);
+
+      mongoc_collection_destroy (ctx.collection);
+      mongoc_client_destroy (client);
+   }
+}
+
+
+
+static void
+test_example_change_stream (mongoc_database_t *db)
+{
+   /* Start Changestream Example 1 */
+   mongoc_collection_t *collection;
+   bson_t pipeline = BSON_INITIALIZER;
+   bson_t opts = BSON_INITIALIZER;
+   mongoc_change_stream_t *stream;
+   const bson_t *change;
+   bson_iter_t iter;
+   bson_error_t error;
+
+   collection = mongoc_database_get_collection (db, "inventory");
+   stream = mongoc_collection_watch (collection, &pipeline, NULL /* opts */);
+   mongoc_change_stream_next (stream, &change);
+   if (mongoc_change_stream_error_document (stream, &error, NULL)) {
+      MONGOC_ERROR ("%s\n", error.message);
+   }
+
+   mongoc_change_stream_destroy (stream);
+   /* End Changestream Example 1 */
+
+   /* Start Changestream Example 2 */
+   BSON_APPEND_UTF8 (&opts, "fullDocument", "updateLookup");
+   stream = mongoc_collection_watch (collection, &pipeline, &opts);
+   mongoc_change_stream_next (stream, &change);
+   if (mongoc_change_stream_error_document (stream, &error, NULL)) {
+      MONGOC_ERROR ("%s\n", error.message);
+   }
+
+   mongoc_change_stream_destroy (stream);
+   /* End Changestream Example 2 */
+
+   bson_reinit (&opts);
+
+   /* Start Changestream Example 3 */
+   stream = mongoc_collection_watch (collection, &pipeline, NULL);
+   if (mongoc_change_stream_next (stream, &change)) {
+      bson_iter_init_find (&iter, change, "_id");
+      BSON_APPEND_VALUE (&opts, "resumeAfter", bson_iter_value (&iter));
+
+      mongoc_change_stream_destroy (stream);
+      stream = mongoc_collection_watch (collection, &pipeline, &opts);
+      mongoc_change_stream_next (stream, &change);
+      mongoc_change_stream_destroy (stream);
+   } else {
+      if (mongoc_change_stream_error_document (stream, &error, NULL)) {
+         MONGOC_ERROR ("%s\n", error.message);
+      }
+
+      mongoc_change_stream_destroy (stream);
+   }
+
+   /* End Changestream Example 3 */
+   bson_destroy (&opts);
+   bson_destroy (&pipeline);
+   bson_destroy (&opts);
+   mongoc_collection_destroy (collection);
+}
+
+
+static void
+test_sample_commands (void)
 {
    mongoc_client_t *client;
    mongoc_database_t *db;
@@ -2538,6 +2670,7 @@ test_sample_commands (void *ctx)
    test_sample_command (test_example_57, 57, db, collection, false);
    test_sample_command (test_example_58, 58, db, collection, false);
    test_sample_command (test_example_56, 56, db, collection, true);
+   test_sample_change_stream_command (test_example_change_stream, db);
 
    mongoc_collection_drop (collection, NULL);
 
@@ -2550,11 +2683,5 @@ test_sample_commands (void *ctx)
 void
 test_samples_install (TestSuite *suite)
 {
-   /* One of the examples uses MongoDB 2.6+'s $currentDate */
-   TestSuite_AddFull (suite,
-                      "/Samples",
-                      test_sample_commands,
-                      NULL,
-                      NULL,
-                      test_framework_skip_if_max_wire_version_less_than_1);
+   TestSuite_AddLive (suite, "/Samples", test_sample_commands);
 }
