@@ -2,18 +2,21 @@
 
 
 #ifdef MONGOC_ENABLE_SSL_OPENSSL
-# include <openssl/err.h>
+#include <openssl/err.h>
 #endif
 
 #include "ssl-test.h"
 #include "TestSuite.h"
 #include "test-libmongoc.h"
 
+#if !defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL) && \
+   !defined(MONGOC_ENABLE_SSL_LIBRESSL)
+
 static void
 test_mongoc_tls_no_certs (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -26,11 +29,12 @@ test_mongoc_tls_no_certs (void)
 }
 
 
+#ifdef MONGOC_ENABLE_SSL_OPENSSL
 static void
 test_mongoc_tls_password (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -50,8 +54,8 @@ test_mongoc_tls_password (void)
 static void
 test_mongoc_tls_bad_password (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -77,13 +81,14 @@ test_mongoc_tls_bad_password (void)
    ASSERT_CMPINT (cr.result, ==, SSL_TEST_SUCCESS);
    ASSERT_CMPINT (sr.result, ==, SSL_TEST_SUCCESS);
 }
+#endif
 
 
 static void
 test_mongoc_tls_no_verify (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -105,8 +110,8 @@ test_mongoc_tls_no_verify (void)
 static void
 test_mongoc_tls_allow_invalid_hostname (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -128,8 +133,8 @@ test_mongoc_tls_allow_invalid_hostname (void)
 static void
 test_mongoc_tls_bad_verify (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -157,8 +162,8 @@ test_mongoc_tls_bad_verify (void)
 static void
 test_mongoc_tls_basic (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -179,8 +184,8 @@ test_mongoc_tls_basic (void)
 static void
 test_mongoc_tls_weak_cert_validation (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -211,8 +216,8 @@ test_mongoc_tls_weak_cert_validation (void)
 static void
 test_mongoc_tls_crl (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -246,8 +251,8 @@ test_mongoc_tls_crl (void)
 static void
 test_mongoc_tls_expired (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -274,8 +279,8 @@ test_mongoc_tls_expired (void)
 static void
 test_mongoc_tls_common_name (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -296,8 +301,8 @@ test_mongoc_tls_common_name (void)
 static void
 test_mongoc_tls_altname (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -318,8 +323,8 @@ test_mongoc_tls_altname (void)
 static void
 test_mongoc_tls_wild (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -336,11 +341,12 @@ test_mongoc_tls_wild (void)
 }
 
 
+#ifdef MONGOC_ENABLE_SSL_OPENSSL
 static void
 test_mongoc_tls_ip (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
@@ -354,19 +360,21 @@ test_mongoc_tls_ip (void)
    ASSERT_CMPINT (cr.result, ==, SSL_TEST_SUCCESS);
    ASSERT_CMPINT (sr.result, ==, SSL_TEST_SUCCESS);
 }
+#endif
 
 
-#if !defined(_WIN32) && defined(MONGOC_ENABLE_SSL_OPENSSL)
+#if !defined(__APPLE__) && !defined(_WIN32) && \
+   defined(MONGOC_ENABLE_SSL_OPENSSL) && OPENSSL_VERSION_NUMBER >= 0x10000000L
 static void
 test_mongoc_tls_trust_dir (void)
 {
-   mongoc_ssl_opt_t sopt = { 0 };
-   mongoc_ssl_opt_t copt = { 0 };
+   mongoc_ssl_opt_t sopt = {0};
+   mongoc_ssl_opt_t copt = {0};
    ssl_test_result_t sr;
    ssl_test_result_t cr;
 
    sopt.pem_file = CERT_SERVER;
-   sopt.ca_dir = CERT_CA;
+   sopt.ca_file = CERT_CA;
 
    copt.ca_dir = CERT_TEST_DIR;
 
@@ -377,15 +385,19 @@ test_mongoc_tls_trust_dir (void)
 }
 #endif
 
+#endif /* !MONGOC_ENABLE_SSL_SECURE_CHANNEL && !MONGOC_ENABLE_SSL_LIBRESSL */
 
 void
 test_stream_tls_install (TestSuite *suite)
 {
-#if !defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL) && !defined(MONGOC_ENABLE_SSL_LIBRESSL)
+#if !defined(MONGOC_ENABLE_SSL_SECURE_CHANNEL) && \
+   !defined(MONGOC_ENABLE_SSL_LIBRESSL)
    TestSuite_Add (suite, "/TLS/commonName", test_mongoc_tls_common_name);
    TestSuite_Add (suite, "/TLS/altname", test_mongoc_tls_altname);
    TestSuite_Add (suite, "/TLS/basic", test_mongoc_tls_basic);
-   TestSuite_Add (suite, "/TLS/allow_invalid_hostname", test_mongoc_tls_allow_invalid_hostname);
+   TestSuite_Add (suite,
+                  "/TLS/allow_invalid_hostname",
+                  test_mongoc_tls_allow_invalid_hostname);
    TestSuite_Add (suite, "/TLS/wild", test_mongoc_tls_wild);
    TestSuite_Add (suite, "/TLS/no_verify", test_mongoc_tls_no_verify);
    TestSuite_Add (suite, "/TLS/bad_verify", test_mongoc_tls_bad_verify);
@@ -397,11 +409,13 @@ test_stream_tls_install (TestSuite *suite)
    TestSuite_Add (suite, "/TLS/ip", test_mongoc_tls_ip);
    TestSuite_Add (suite, "/TLS/password", test_mongoc_tls_password);
    TestSuite_Add (suite, "/TLS/bad_password", test_mongoc_tls_bad_password);
-   TestSuite_Add (suite, "/TLS/weak_cert_validation", test_mongoc_tls_weak_cert_validation);
+   TestSuite_Add (
+      suite, "/TLS/weak_cert_validation", test_mongoc_tls_weak_cert_validation);
    TestSuite_Add (suite, "/TLS/crl", test_mongoc_tls_crl);
 #endif
 
-#if !defined(__APPLE__) && !defined(_WIN32) && defined(MONGOC_ENABLE_SSL_OPENSSL)
+#if !defined(__APPLE__) && !defined(_WIN32) && \
+   defined(MONGOC_ENABLE_SSL_OPENSSL) && OPENSSL_VERSION_NUMBER >= 0x10000000L
    TestSuite_Add (suite, "/TLS/trust_dir", test_mongoc_tls_trust_dir);
 #endif
 #endif
