@@ -1,5 +1,6 @@
 #include <mongoc.h>
 #include <mongoc-write-concern-private.h>
+#include <mongoc-util-private.h>
 
 #include "TestSuite.h"
 #include "test-conveniences.h"
@@ -19,8 +20,12 @@ test_write_concern_append (void)
    wc = NULL;
    BSON_ASSERT (!mongoc_write_concern_append (wc, cmd));
 
-   /* append valid writeConcern */
+   /* append default writeConcern */
    wc = mongoc_write_concern_new ();
+   ASSERT (mongoc_write_concern_is_default (wc));
+   ASSERT_MATCH (cmd, "{'foo': 1, 'writeConcern': {'$exists': false}}");
+
+   /* append writeConcern with w */
    mongoc_write_concern_set_w (wc, 1);
    BSON_ASSERT (mongoc_write_concern_append (wc, cmd));
 
@@ -34,7 +39,6 @@ static void
 test_write_concern_basic (void)
 {
    mongoc_write_concern_t *write_concern;
-   const bson_t *gle;
    const bson_t *bson;
    bson_iter_t iter;
 
@@ -86,26 +90,15 @@ test_write_concern_basic (void)
     */
    mongoc_write_concern_set_fsync (write_concern, true);
    mongoc_write_concern_set_journal (write_concern, true);
-   gle = _mongoc_write_concern_get_gle (write_concern);
-   ASSERT (bson_iter_init_find (&iter, gle, "getlasterror") &&
-           BSON_ITER_HOLDS_INT32 (&iter) && bson_iter_int32 (&iter) == 1);
-   ASSERT (bson_iter_init_find (&iter, gle, "fsync") &&
-           BSON_ITER_HOLDS_BOOL (&iter) && bson_iter_bool (&iter));
-   ASSERT (bson_iter_init_find (&iter, gle, "j") &&
-           BSON_ITER_HOLDS_BOOL (&iter) && bson_iter_bool (&iter));
-   ASSERT (bson_iter_init_find (&iter, gle, "w") &&
-           BSON_ITER_HOLDS_INT32 (&iter) && bson_iter_int32 (&iter) == 3);
-   ASSERT (gle);
 
    bson = _mongoc_write_concern_get_bson (write_concern);
-   ASSERT (!bson_iter_init_find (&iter, bson, "getlasterror"));
+   ASSERT (bson);
    ASSERT (bson_iter_init_find (&iter, bson, "fsync") &&
            BSON_ITER_HOLDS_BOOL (&iter) && bson_iter_bool (&iter));
    ASSERT (bson_iter_init_find (&iter, bson, "j") &&
            BSON_ITER_HOLDS_BOOL (&iter) && bson_iter_bool (&iter));
    ASSERT (bson_iter_init_find (&iter, bson, "w") &&
            BSON_ITER_HOLDS_INT32 (&iter) && bson_iter_int32 (&iter) == 3);
-   ASSERT (bson);
 
    mongoc_write_concern_destroy (write_concern);
 
@@ -117,7 +110,6 @@ static void
 test_write_concern_bson_omits_defaults (void)
 {
    mongoc_write_concern_t *write_concern;
-   const bson_t *gle;
    const bson_t *bson;
    bson_iter_t iter;
 
@@ -128,20 +120,10 @@ test_write_concern_bson_omits_defaults (void)
     */
    ASSERT (write_concern);
 
-   gle = _mongoc_write_concern_get_gle (write_concern);
-   ASSERT (bson_iter_init_find (&iter, gle, "getlasterror") &&
-           BSON_ITER_HOLDS_INT32 (&iter) && bson_iter_int32 (&iter) == 1);
-   ASSERT (!bson_iter_init_find (&iter, gle, "fsync"));
-   ASSERT (!bson_iter_init_find (&iter, gle, "j"));
-   ASSERT (!bson_iter_init_find (&iter, gle, "w"));
-   ASSERT (gle);
-
    bson = _mongoc_write_concern_get_bson (write_concern);
-   ASSERT (!bson_iter_init_find (&iter, bson, "getlasterror"));
+   ASSERT (bson);
    ASSERT (!bson_iter_init_find (&iter, bson, "fsync"));
    ASSERT (!bson_iter_init_find (&iter, bson, "j"));
-   ASSERT (!bson_iter_init_find (&iter, gle, "w"));
-   ASSERT (bson);
 
    mongoc_write_concern_destroy (write_concern);
 }
@@ -151,7 +133,6 @@ static void
 test_write_concern_bson_includes_false_fsync_and_journal (void)
 {
    mongoc_write_concern_t *write_concern;
-   const bson_t *gle;
    const bson_t *bson;
    bson_iter_t iter;
 
@@ -164,36 +145,25 @@ test_write_concern_bson_includes_false_fsync_and_journal (void)
    mongoc_write_concern_set_fsync (write_concern, false);
    mongoc_write_concern_set_journal (write_concern, false);
 
-   gle = _mongoc_write_concern_get_gle (write_concern);
-   ASSERT (bson_iter_init_find (&iter, gle, "getlasterror") &&
-           BSON_ITER_HOLDS_INT32 (&iter) && bson_iter_int32 (&iter) == 1);
-   ASSERT (bson_iter_init_find (&iter, gle, "fsync") &&
-           BSON_ITER_HOLDS_BOOL (&iter) && !bson_iter_bool (&iter));
-   ASSERT (bson_iter_init_find (&iter, gle, "j") &&
-           BSON_ITER_HOLDS_BOOL (&iter) && !bson_iter_bool (&iter));
-   ASSERT (!bson_iter_init_find (&iter, gle, "w"));
-   ASSERT (gle);
-
    bson = _mongoc_write_concern_get_bson (write_concern);
-   ASSERT (!bson_iter_init_find (&iter, bson, "getlasterror"));
+   ASSERT (bson);
    ASSERT (bson_iter_init_find (&iter, bson, "fsync") &&
            BSON_ITER_HOLDS_BOOL (&iter) && !bson_iter_bool (&iter));
    ASSERT (bson_iter_init_find (&iter, bson, "j") &&
            BSON_ITER_HOLDS_BOOL (&iter) && !bson_iter_bool (&iter));
    ASSERT (!bson_iter_init_find (&iter, bson, "w"));
-   ASSERT (bson);
 
    mongoc_write_concern_destroy (write_concern);
 }
 
 
 static void
-test_write_concern_fsync_and_journal_gle_and_validity (void)
+test_write_concern_fsync_and_journal_w1_and_validity (void)
 {
    mongoc_write_concern_t *write_concern = mongoc_write_concern_new ();
 
    /*
-    * Journal and fsync should imply GLE regardless of w; however, journal and
+    * Journal and fsync should imply w=1 regardless of w; however, journal and
     * fsync logically conflict with w=0 and w=-1, so a write concern with such
     * a combination of options will be considered invalid.
     */
@@ -304,12 +274,13 @@ test_write_concern_wtimeout_validity (void)
 }
 
 static void
-_test_write_concern_from_iterator (const char *swc, bool ok)
+_test_write_concern_from_iterator (const char *swc, bool ok, bool is_default)
 {
    bson_t *bson = tmp_bson (swc);
    const bson_t *bson2;
    mongoc_write_concern_t *wc;
    bson_iter_t iter;
+   bson_error_t error;
 
    if (test_suite_debug_output ()) {
       fprintf (stdout, "  - %s\n", swc);
@@ -317,64 +288,121 @@ _test_write_concern_from_iterator (const char *swc, bool ok)
    }
 
    bson_iter_init_find (&iter, bson, "writeConcern");
-   ASSERT_CMPINT (
-      (int) _mongoc_write_concern_iter_is_valid (&iter), ==, (int) ok);
-
-   wc = _mongoc_write_concern_new_from_iter (&iter);
-   bson2 = _mongoc_write_concern_get_bson (wc);
-   ASSERT (bson_compare (bson, bson2));
-   mongoc_write_concern_destroy (wc);
+   wc = _mongoc_write_concern_new_from_iter (&iter, &error);
+   if (ok) {
+      BSON_ASSERT (wc);
+      ASSERT (mongoc_write_concern_is_default (wc) == is_default);
+      bson2 = _mongoc_write_concern_get_bson (wc);
+      ASSERT (bson_compare (bson, bson2));
+      mongoc_write_concern_destroy (wc);
+   } else {
+      BSON_ASSERT (!wc);
+      ASSERT_ERROR_CONTAINS (error,
+                             MONGOC_ERROR_COMMAND,
+                             MONGOC_ERROR_COMMAND_INVALID_ARG,
+                             "Invalid writeConcern");
+   }
 }
 
 static void
 test_write_concern_from_iterator (void)
 {
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 'majority'}}",
-                                      true);
+   _test_write_concern_from_iterator ("{'writeConcern': {}}", true, true);
    _test_write_concern_from_iterator (
-      "{'writeConcern': {'w': 'majority', 'j': true}}", true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 'sometag'}}",
-                                      true);
+      "{'writeConcern': {'w': 'majority'}}", true, false);
    _test_write_concern_from_iterator (
-      "{'writeConcern': {'w': 'sometag', 'j': true}}", true);
+      "{'writeConcern': {'w': 'majority', 'j': true}}", true, false);
    _test_write_concern_from_iterator (
-      "{'writeConcern': {'w': 'sometag', 'j': false}}", true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 1, 'j': true}}",
-                                      true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 1, 'j': false}}",
-                                      true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 0, 'j': true}}",
-                                      false);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 0, 'j': false}}",
-                                      true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 42}}", true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 1}}", true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'j': true}}", true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'j': false}}", true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': -3}}", true);
+      "{'writeConcern': {'w': 'sometag'}}", true, false);
    _test_write_concern_from_iterator (
-      "{'writeConcern': {'w': 'majority', 'wtimeout': 42}}", true);
+      "{'writeConcern': {'w': 'sometag', 'j': true}}", true, false);
    _test_write_concern_from_iterator (
-      "{'writeConcern': {'w': 'sometag', 'wtimeout': 42}}", true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'wtimeout': 42}}",
-                                      true);
+      "{'writeConcern': {'w': 'sometag', 'j': false}}", true, false);
    _test_write_concern_from_iterator (
-      "{'writeConcern': {'w': 1, 'wtimeout': 42}}", true);
+      "{'writeConcern': {'w': 1, 'j': true}}", true, false);
    _test_write_concern_from_iterator (
-      "{'writeConcern': {'w': 0, 'wtimeout': 42}}", true);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': 1.0}}", false);
+      "{'writeConcern': {'w': 1, 'j': false}}", true, false);
    _test_write_concern_from_iterator (
-      "{'writeConcern': {'w': {'some': 'stuff'}}}", false);
-   _test_write_concern_from_iterator ("{'writeConcern': {'w': []}}", false);
-   _test_write_concern_from_iterator ("{'writeConcern': {'wtimeout': 'never'}}",
-                                      false);
-   _test_write_concern_from_iterator ("{'writeConcern': {'j': 'never'}}",
-                                      false);
-   _test_write_concern_from_iterator ("{'writeConcern': {'j': 1.0}}", false);
-   _test_write_concern_from_iterator ("{'writeConcern': {'fsync': 1.0}}",
-                                      false);
-   _test_write_concern_from_iterator ("{'writeConcern': {'fsync': true}}",
-                                      true);
+      "{'writeConcern': {'w': 0, 'j': true}}", false, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': 0, 'j': false}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': 42}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': 1}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'j': true}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'j': false}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': -2}}", true, true);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': -3}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': 'majority', 'wtimeout': 42}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': 'sometag', 'wtimeout': 42}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'wtimeout': 42}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': 1, 'wtimeout': 42}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': 0, 'wtimeout': 42}}", true, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': 1.0}}", false, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': {'some': 'stuff'}}}", false, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'w': []}}", false, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'wtimeout': 'never'}}", false, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'j': 'never'}}", false, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'j': 1.0}}", false, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'fsync': 1.0}}", false, false);
+   _test_write_concern_from_iterator (
+      "{'writeConcern': {'fsync': true}}", true, false);
+}
+
+
+static void
+test_write_concern_always_mutable (void)
+{
+   mongoc_write_concern_t *write_concern;
+
+   write_concern = mongoc_write_concern_new ();
+
+   ASSERT (write_concern);
+
+   mongoc_write_concern_set_fsync (write_concern, true);
+   ASSERT_MATCH (_mongoc_write_concern_get_bson (write_concern),
+                 "{'fsync': true}");
+
+   mongoc_write_concern_set_journal (write_concern, true);
+   ASSERT_MATCH (_mongoc_write_concern_get_bson (write_concern),
+                 "{'fsync': true, 'j': true}");
+
+   mongoc_write_concern_set_w (write_concern, 2);
+   ASSERT_MATCH (_mongoc_write_concern_get_bson (write_concern),
+                 "{'w': 2, 'fsync': true, 'j': true}");
+
+   mongoc_write_concern_set_wtimeout (write_concern, 100);
+   ASSERT_MATCH (_mongoc_write_concern_get_bson (write_concern),
+                 "{'w': 2, 'fsync': true, 'j': true, 'wtimeout': 100}");
+
+   mongoc_write_concern_set_wmajority (write_concern, 200);
+   ASSERT_MATCH (
+      _mongoc_write_concern_get_bson (write_concern),
+      "{'w': 'majority', 'fsync': true, 'j': true, 'wtimeout': 200}");
+
+   mongoc_write_concern_set_wtag (write_concern, "MultipleDC");
+   ASSERT_MATCH (
+      _mongoc_write_concern_get_bson (write_concern),
+      "{'w': 'MultipleDC', 'fsync': true, 'j': true, 'wtimeout': 200}");
+
+   mongoc_write_concern_destroy (write_concern);
 }
 
 
@@ -391,10 +419,12 @@ test_write_concern_install (TestSuite *suite)
                   test_write_concern_bson_includes_false_fsync_and_journal);
    TestSuite_Add (suite,
                   "/WriteConcern/fsync_and_journal_gle_and_validity",
-                  test_write_concern_fsync_and_journal_gle_and_validity);
+                  test_write_concern_fsync_and_journal_w1_and_validity);
    TestSuite_Add (suite,
                   "/WriteConcern/wtimeout_validity",
                   test_write_concern_wtimeout_validity);
    TestSuite_Add (
       suite, "/WriteConcern/from_iterator", test_write_concern_from_iterator);
+   TestSuite_Add (
+      suite, "/WriteConcern/always_mutable", test_write_concern_always_mutable);
 }
